###################################################################################################################################

#Auxiliary functions 

 #Overview:
  #Line 0017 - 0141: Custom Bootstrapping function
  #Line 0142 - 0191: Function to define output of each bootstrapping run
  #Line 0192 - 0228: Function to transform the output of the bootstrap from a list into a dataframe
  #Line 0229 - 0245: Custom function to calculate quantile or bca confidence intervals
  #Line 0246 - 0255: Custom ggplot theme function

  #Contact: tom.versluijs@gmail.com
  #Date: 2022/11/01

###################################################################################################################################

#(I): Bootstrapping function, partly based on packages boot and car
f_bootstrap_Tom <- function(mod=mod, mod_func=mod_func, data=data, data_x="x" , data_y="y", newdata=newdata, 
                            nsim=10000, boot_type=c("fixef", "residual", "case")){
  
  #(1) METHOD 1: Re-sample fixed parameter values from multivariate normal distribution based on the original 
  #estimates and their variance-covariance matrix
  if(boot_type=="fixef"){
    
    #Re-sample fixed parameter values:
    pars.picked <- MASS::mvrnorm(nsim, mu = coef(mod), Sigma = vcov(mod))
    #For each newly sampled combination of parameters calculate the predicted y value at each newdata value
    df_fitted <- data.frame(apply(pars.picked, 1, function(x){mod_func(newdata[,data_x], a=x[1], b=x[2])}))
    
    #Restructure pars.picked
    df_tmp <- data.frame(t(pars.picked))
    row.names(df_tmp) <- c("a", "b")
     
    #Check how many models did not converge
    converged <- sum(!sapply(df_fitted, is.null))   
    print(paste0(converged, "  out of ", nsim, " converged"))
    
  }
  
  #(2) METHOD 2: Residual bootstrapping
  if(boot_type=="residual"){
    
    #Note that residual bootstrapping currently only works with linear models (lm) and when the response variable is un-transformed, or is log-transformed
    
    #Specify the residual re-sampling function
    sampfun_resid <- function(mod, data, data_x, data_y) {
      
      #Extract fitted values of original model (Note: predictions are on the scale of the response variable of 'mod'):
      pp <- predict(mod, newdata=data)
      
      #Extract standardized Pearson residuals from original model
      rsds.std <- residuals(mod, type="pearson")
      rsds.sigma <- attr(mod[["residuals"]], "std")
      
      if(!is.null(rsds.sigma)){rr <- rsds.std * rsds.sigma}
      if(is.null(rsds.sigma)){rr <- rsds.std}
      
      #Center the residuals (van der Leeden  et al 2007)
      rr <- rr - mean(rr)
      
      #Create a dataframe where we combine original data with fitted values and residuals 
      res <- data.frame(data, pred=pp, rr=rr)
      
      #Re-sample residuals with replacement
      res_resid <- sample(res$rr, size=nrow(res), replace=TRUE)
      
      #Create bootstrapped predicted values by adding bootstrapped residuals to the original predicted values
      res[,data_y] <- res$pred + res_resid
  
      #Check if response variable is log-transformed, and if so transform predictions back to the arithmetic scale
      if(grepl(paste0("lm, log(", data_y, ")"), toString(mod$call), fixed = TRUE)){
        res[,data_y] <- exp(res[,data_y])
        }
      
      #Return dataframe res as output
      return(res)
    }
    
    #Conduct residual bootstrapping by iterating the following process 1000 times: (I) use sampfun_resid to generate newly sampled datasets by
    #resampling of residuals, (II) refit the model to this dataset and run function 'boot_func' to extract model coefficients and predictions.
    boot_out <- replicate(nsim, 
                          suppressWarnings(f_boot_out(mod = update(mod, 
                                                                   data = sampfun_resid(mod=mod, data=data, data_x=data_x, data_y=data_y)),
                                                      mod_func = mod_func,
                                                      newdata = newdata,
                                                      data_x = data_x)))
  }
  
  #(3) METHOD 3: Case re-sampling
  if(boot_type=="case"){
    
    #Specify the case re-sampling function
    sampfun_case <- function(data) {
      
      #(I) Re-sample dataframe rows (cases) with replacement
      data_bootcase <- data[sample(nrow(data), replace=TRUE),]
      return(data_bootcase)
      
    }
    
    #Conduct hierarchical bootstrapping by iterating the following process nsim times: (I) use sampfun_case to generate newly sampled  
    #datasets, (II) refit the model to this dataset and run function 'f_boot_out' to extract the statistics of interest (e.g. model 
    #predictions, model coefficients and/or smearing factor)
    boot_out <- replicate(nsim, 
                          suppressWarnings(f_boot_out(mod=update(mod, 
                                                                 data = sampfun_case(data=data)),
                                                      mod_func = mod_func,
                                                      newdata = newdata)))
    
  }
  
  #Boot_out is a list that contains separate sublists with bootstrapping runs for all statistics of interest. These
  #parameters are: 'a' (intercept), 'b' (slope), 'SF' (smearing factor) and 'fitted' (predicted values). We first 
  #transform each sublist to a separate dataframe with bootstrapping runs as separate columns.
  
  #Transform each sublist to a dataframe (using the auxiliary function 'f_list_to_df')
  if(boot_type!="fixef"){
    
    #Transform bootstrapped list elements to dataframes
    df_a <- f_list_to_df(boot_out['a',])  
    df_b <- f_list_to_df(boot_out['b',])
    df_sf <- f_list_to_df(boot_out['SF',]) 
    df_tmp <- rbind(df_a, df_b, df_sf) ; rm(df_a, df_b, df_sf)
    df_fitted <- f_list_to_df(boot_out['fitted',]) 
      
    #Check how many models did not converge
    converged <- sum(!sapply(boot_out['fitted',], is.null))   
    print(paste0(converged, "  out of ", nsim, " converged"))
    
  }
  
  #Store the output from all bootstrapping runs in the list 'l_boot_out'
  l_boot_out <- list("coefficients" = df_tmp, 
                     "fitted" = df_fitted, 
                     "converged" = converged,
                     "nsim" = nsim)
  
  #Pass 'l_boot_out' as output of the bootstrapping function  
  return(l_boot_out)
}

#(II): Function to calculate several statistics of interest as output for each bootstrapping run (outputs a list)
f_boot_out <- function(mod, newdata, data_x, mod_func, ...) {
  
  #Trycatch ensures that the code continues even if a model does not converge or has singularity issues
  tryCatch({
    
    #Extract model coefficients
    a <- coef(mod)[1]  ;  names(a) <- "a"
    b <- coef(mod)[2]  ;  names(b) <- "b"
    
    #Extract model predictions
    fitted <- mod_func(newdata[,data_x], a=a, b=b)
    fitted <- setNames(fitted, 1:length(fitted))
    
    #If the response variable is log-transformed, calculate Duan's smearing factor and store coefficients and fitted values in a list
    if(grepl(paste0("lm, log(", data_y, ")"), toString(mod$call), fixed = TRUE)){
      
       #Calculate Duan's smearing factor 
       SF_mod <- mean(exp(residuals(mod)))
       names(SF_mod) <- "SF"
      
       #Store all elements in a list
       list_boot_tmp <- list("a" = a, 
                             "b" = b, 
                             "fitted" = fitted, 
                             "SF" = SF_mod)
      }
    
    #If the response variable is not log-transformed, set Duan's smearing factor to NA and store coefficients and fitted values in a list
    if(!grepl(paste0("lm, log(", data_y, ")"), toString(mod$call), fixed = TRUE)){
      
      #Set Duan's smearing factor to NA
      SF_mod <- NA
      names(SF_mod) <- "SF"
      
      #Store all elements in a list
      list_boot_tmp <- list("a" = a, 
                            "b" = b, 
                            "fitted" = fitted, 
                            "SF" = SF_mod)
      
      }
      
    #Give list as output
    return(list_boot_tmp)
    
  }, error=function(e){cat("ERROR :",conditionMessage(e), "\n")} )
  
}

#(III): Function to transform a list of bootstrapping runs into a dataframe
f_list_to_df <- function(boot_out, ...){
  
  #Remove null elements from list (i.e. runs where the model did not converge)
  boot_out <- boot_out[!sapply(boot_out, is.null)]
  
  #Transform list to matrix while preserving row.names
  
  #Check if all list elements are named
  check <- all(unlist(lapply(boot_out, FUN=function(x){!is.null(names(x))})))
  
  #Merge named list elements into a matrix
  if(check==T){
    
    #Create a vector of all unique list level names:
    element_names <- unique(unlist(lapply(boot_out, names)))
    
    #For each list element, select all 'element_names'. If the element does not occur in the current list then NA will be added.
    boot_out <- lapply(boot_out, FUN=function(x){x[element_names]})
    
    #Combine list elements into a transposed matrix
    df <- data.frame(matrix(unlist(boot_out), nrow=length(boot_out), byrow=TRUE))
    names(df) <- element_names
    boot_out <- t(df)
    
  }
  if(check==F){
    print("Error: Make sure that the specified 'boot_func' gives a named vector as output")
  }
  
  #Transform matrix to dataframe
  boot_out <- data.frame(boot_out)
  
  #Return dataframe as output
  return(boot_out)
}

#(IV): Function to calculate confidence intervals over all bootstrap runs
get_CI <- function(y, pref="", type=c("quantile", "bca"), boot_confint=0.95, ...) {
  
  if(type=="quantile"){
    lwr_ci <- (1-boot_confint)/2
    r1 <- t(apply(y, 1, FUN=function(x){quantile(na.omit(x), probs=c(lwr_ci, 1-lwr_ci))} ))
    r1 <- setNames(as.data.frame(r1), paste0(pref, "_", type, c("_lci","_uci")))
    return(r1)
  }
  
  if(type=="bca"){
    r1 <- t(apply(y, 1, FUN=function(x){coxed::bca(na.omit(x), conf.level=boot_confint)} ))
    r1 <- setNames(as.data.frame(r1), paste0(pref, "_", type, c("_lci","_uci")))
    return(r1)
  }
}  

#(V): Custom ggplot theme function
theme_tom <- function(){
  theme_classic() %+replace%
    theme(axis.title = element_text(size=16),
          axis.text = element_text(size=14),
          legend.position = "none",
          strip.background = element_rect(fill = "white", colour = "black", size = rel(2)),
          complete = TRUE,
          title=element_text(size=15, face="bold"))}

###################################################################################################################################
