
##############################################################################################################################################

#(2): Data preparation of Zackenberg piftall dataset 1996 - 2020

#Overview:
#Line 0014 - 0061: Initialize workspace and load data
#Line 0062 - 0174: Minor data adjustments
#Line 0175 - 0305: Major data adjustments

#Contact: tom.versluijs@gmail.com
#Date: 2022/11/01

####################################################################################################################################

#(I): Initialize workspace and load data

####################################################################################################################################

 #Clear Workspace
  rm(list=ls())

 #Load data:
  df <- read.csv("Versluijs2022_Input_2-ArthropodDataZAC_DataPreparation_RawDataZAC.csv", header=T, sep='\t')
  #Note, this corresponds to file "View_BioBasis_Zackenberg_Data_Arthropods_Arthropod_emergence120920221646373401.csv" downloaded on 12-09-2022.
  #This dataset is freely available and can be downloaded from the following link: https://data.g-e-m.dk/datasets?doi=10.17897/V285-Z265.

  #METADATA:
   #Date = Date of sampling
   #Hour = Start hour of sampling (UTC)
   #PlotID = Plot ID
   #Sorter = Name of labworker(s)
   #Days A = Number of days the traps have been open in section A, or (in 1996) sections A-D combined. For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
   #Days B = Number of days the traps have been open in section B, or (in 1996) sections A-D combined. For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
   #Days C = Number of days the traps have been open in section C, or (in 1996) sections A-D combined. For Art1, C refers to the East chamber (2006-13).
   #Days D = Number of days the traps have been open in section D, or (in 1996) sections A-D combined. For Art1, D refers to the West chamber (2006-13).
   #Days E = Number of days the traps have been open in section E.
   #Days F = Number of days the traps have been open in section F.
   #Days G = Number of days the traps have been open in section G.
   #Days H = Number of days the traps have been open in section H.
   #Phylum = Phylum of the collected arthropod
   #Order = Order of the collected arthropod
   #Family = Family or super-family of the collected arthropod
   #Genus = Genus of the collected arthropod
   #Species = Species of the collected arthropod
   #A = The number of collected individuals from section A, or (in 1996) sections A-D combined.  For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
   #B = The number of collected individuals from section B, or (in 1996) sections A-D combined.  For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
   #C = The number of collected individuals from section C, or (in 1996) sections A-D combined.  For Art1, C refers to the East chamber (2006-13).
   #D = The number of collected individuals from section D, or (in 1996) sections A-D combined.  For Art1, D refers to the West chamber (2006-13).
   #E = The number of collected individuals from section E.
   #F = The number of collected individuals from section F.
   #G = The number of collected individuals from section G.
   #H = The number of collected individuals from section H.
   #Field_remarks = Remarks from the field observer
   #Sorting_remarks = Remarks from the sorter
   #General_remarks = General remarks

 #Load packages
  library(dplyr)
  library(ggplot2)
 
####################################################################################################################################
 
#(II): Minor data adjustments
 
####################################################################################################################################
 
#(A): Assess dataframe
  str(df)
  colnames(df)[1] <- "Date"
  colnames(df)[2] <- "Time"
  colnames(df)[colnames(df) %in% c("A", "B", "C", "D", "E", "F", "G", "H")] <- c("Plot_A", "Plot_B", "Plot_C", "Plot_D", "Plot_E", "Plot_F", "Plot_G", "Plot_H")
  df$Date <- as.Date(df$Date)
 
#(B): Change date and time into a datetime column
 
 #If time is not specified, set to noon
  df$Time[is.na(df$Time)] <- 12
  df$Time[df$Time < 0] <- 12
  
 #Change time annotation:  
  df$Time[df$Time<10 & !is.na(df$Time)] <- paste0(0, df$Time[df$Time<10 & !is.na(df$Time)] )
  df$Time[!is.na(df$Time)] <-  paste0(df$Time[!is.na(df$Time)], ":00:00")
 
 #Create datetime:  
  df$datetime <- paste0(df$Date, " ", df$Time)
  df$datetime <- as.POSIXct(df$datetime, tz="GMT", format = "%Y-%m-%d %H:%M:%S")
 
#(C): Replace -9999 or -999 by NA:
  df[df==-9999] <- NA
  df[df==-999] <- NA

#(D): Adjust 1996 data
   
 #IMPORTANT: Note that the columns Days.A - Days.H, and Plot_A - Plot_H indicate the number of days the traps have been open in each section,
 #and the number of collected individuals in each section respectively. However, there is an exception for 1996 where the data for sections
 #A-D have been combined (Both the Days and Plot columns). To correct for this we divide the values in both columns by 4 and allocate these 
 #values to the columns for A-D.

 #Adjust 1996 data:
  index_1996 <- grepl( "1996", df$Date, fixed = TRUE)
  tmp_days <- round(df[index_1996, "Days.A"] / 4, 0) #make sure these are integers
  tmp_counts <- as.numeric(df[index_1996, "Plot_A"] / 4) #double precision
  df[index_1996, "Days.A"] <- tmp_days
  df[index_1996, "Days.B"] <- tmp_days
  df[index_1996, "Days.C"] <- tmp_days
  df[index_1996, "Days.D"] <- tmp_days
  df[index_1996, "Plot_A"] <- tmp_counts
  df[index_1996, "Plot_B"] <- tmp_counts
  df[index_1996, "Plot_C"] <- tmp_counts
  df[index_1996, "Plot_D"] <- tmp_counts
  rm(index_1996, tmp_days, tmp_counts)

#(E): Adjust 2010 data   
  
  #IMPORTANT: Adjust 2010 data because for a few rows traps B and C have been lumped together into B, and trap days were summed (7+7=14). 
  #We split the trap days and caught arthropods in the respective columns.
 
  #Adjust trap days:
  df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Days.C"] <- 
    df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Days.B"] / 2
  df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Days.B"] <- 
    df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Days.B"] / 2
 
  #Adjust caught arthropods (equally divide the number in Plot_B over the columns Plot_B and Plot_C, but only allow for integers)
  art_tmp <- df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Plot_B"] / 2
  set.seed(23)
  random <- runif(length(art_tmp),0,1)
  index <- (random > 0.5) & (art_tmp != 0)
  index2 <- (random <= 0.5) & (art_tmp != 0)
  art_tmp[index] <- ceiling(art_tmp[index])
  art_tmp[index2] <- floor(art_tmp[index2])
  df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Plot_C"] <- art_tmp
  df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Plot_B"] <- 
   (df[df$Sorting_remarks %in% "Samples B & C lumped together by mistake. Trap days corrected accordingly (NMS)", "Plot_B"] - art_tmp)
  rm(art_tmp, random, index, index2)

#(F): Exclude rows where none of the traps were open (NA, or 0 days), or arthropods were not counted (NA):
   
   #There are several occasions where the number of arthropods was counted (e.g. 0), but the corresponding 
   #trap hasn't been opened (i.e. Days == NA, or Days == 0). We can only use occasions where both counts of arthropods 
   #were made and where we're able to correct for the number of days a trap was active. If either of these is NA, then
   #the data can thus not be used. (Note that there are no counts > 0 that do not have the corresponding trap-days
   #data, so we are not excluding any relevant data here). 

   #Specify per trap if counts of arthropods AND info on trap-days (>0) are available: 
   df$use.A <- !is.na(df$Days.A) & !is.na(df$Plot_A) & df$Days.A!=0
   df$use.B <- !is.na(df$Days.B) & !is.na(df$Plot_B) & df$Days.B!=0
   df$use.C <- !is.na(df$Days.C) & !is.na(df$Plot_C) & df$Days.C!=0
   df$use.D <- !is.na(df$Days.D) & !is.na(df$Plot_D) & df$Days.D!=0
   df$use.E <- !is.na(df$Days.E) & !is.na(df$Plot_E) & df$Days.E!=0
   df$use.F <- !is.na(df$Days.F) & !is.na(df$Plot_F) & df$Days.F!=0
   df$use.G <- !is.na(df$Days.G) & !is.na(df$Plot_G) & df$Days.G!=0
   df$use.H <- !is.na(df$Days.H) & !is.na(df$Plot_H) & df$Days.H!=0

   #keep the row in the dataframe if any of the traps contains ccounts of arthropos ANd info on trap-days
   df <- df %>%
     rowwise() %>%
     mutate(keep=any(use.A, use.B, use.C, use.D, use.E, use.F, use.G, use.H))
  
   #Exclude rows where none of the traps were open, or insects were not counted:
   #tmp <- df[df$keep==FALSE,]
   df <- df[df$keep==TRUE,]
  
   #Exclude unused columns:
   use <- df[,c("use.A", "use.B", "use.C", "use.D", "use.E", "use.F", "use.G", "use.H")]
   df <- df[,c("datetime", "Plot.ID", "Days.A", "Days.B", "Days.C", "Days.D", "Days.E", "Days.F", "Days.G", "Days.H",
               "Phylum", "Order", "Family", "Genus", "Species", "Plot_A", "Plot_B", "Plot_C", "Plot_D", "Plot_E", 
               "Plot_F", "Plot_G", "Plot_H")] 
  
   #Change columns with counts to numeric:
   df[c("Plot_A", "Plot_B", "Plot_C", "Plot_D", "Plot_E", "Plot_F", "Plot_G", "Plot_H")] <- 
    sapply(df[c("Plot_A", "Plot_B", "Plot_C", "Plot_D", "Plot_E", "Plot_F", "Plot_G", "Plot_H")], as.numeric)
  
####################################################################################################################################
   
#(III): Major data adjustments
   
####################################################################################################################################
   
 #The datetime depicted in 'df' represents the date of pitfall collection.However the collected arthropods represent a cumulative
 #number over specimens caught in the period of x-days before this collection date. It would thus be better to spread out this
 #number over all the days a trap was active. To do so, We first check per row in 'df' for how many days the sampling took place 
 #and create new rows for all those days. We then divide the total number of arthropods caught by the number of sampling days, 
 #and allocate each average value to all of these sampling days. For example, if 70 specimens were caught in a trap that was 
 #open for 7 days and that was emptied at day of year 150, then create new rows in the dataframe for day of year 144 - 150 and 
 #allocate 7 specimens to each row. We store everything in a list to speed up processing speed.

  #Create an empty list:
   df_list <- list()

  #Iterate through all rows of the dataframe and create separate entries for each day that traps were open:  
   for(i in 1:nrow(df)){
    
    #for debugging: 
     #i=2964
     
    #Create a temporary empty dataframe:  
     df_tmp <- df[0,]
 
    #Store the row for which all steps have to be conducted:    
     tmp_row <- df[i,]
    
    #Create an index parameter that shows which columns both have arthropod counts and trap-days registered: 
     use_column <- use[i,]
     index <- which(use_column==TRUE)+2
    
    #Store sum of counts before processing (for debugging)
     sum_before <- sum(tmp_row[1, index+13] %>% replace(is.na(.), 0))

    #what is the maximum number of trap days among all traps of this row of the dataframe: 
     max_trap_days <- max(tmp_row[,index])
     
    #Create extra rows containing the datetime for all trap days: 
     for(c in 1:max_trap_days){
       row <- max_trap_days - c
       df_tmp[c, 'datetime'] <- tmp_row$datetime - (row*60*60*24)
      }
    
    #update all other rows that should contain the same information as our starting row:
     df_tmp[1:max_trap_days, c("Plot.ID", "Phylum", "Order", "Family", "Genus", "Species")] <-
       tmp_row[rep(1, each = max_trap_days), c("Plot.ID", "Phylum", "Order", "Family", "Genus", "Species")]
    
    #Add the average number of arthropods caught for each separate trapping day (use average values):
    
     #Select columns that have arthropod counts:
     for(r in index){
       
       #Determine number of trap-days
        days <- tmp_row[,r]
    
       #If number of trap-days is 1, then do not take an average
        if(days==1){
         df_tmp[max_trap_days, r+13] <- tmp_row[,r+13]
         df_tmp[max_trap_days, r] <- tmp_row[,r]
         }
        
       #If number of trap days is larger than 1, then take averages for each trap-day:  
        if(days>1){
         df_tmp[as.numeric(max_trap_days-days+1):max_trap_days, r+13] <- as.numeric(rep(tmp_row[, r+13], days))/days
         df_tmp[max_trap_days, r] <- tmp_row[,r]
         }
        
     }

    #Store sum of counts after processing (for debugging)
     sum_after <- sum(df_tmp[1:max_trap_days, index+13] %>% replace(is.na(.), 0))
     
    #Check if sum_before==sum_after
     if(round(sum_before, 0)!=round(sum_after, 0)){
       print("Stop! Sums not equal")
       print(i)
       break
       }

    #Store dataframe in the pre-specified list:
      df_list[[i]] <- df_tmp
     
    #Print progress:
     print(paste0(round((100*i)/nrow(df),3), " %"))
    
  }
  
  #Combine list into dataframe:
   df_2 = do.call(rbind, df_list)
  
  #remove obsolete variables:
   rm(days, df_tmp, tmp_row, use_column, c, i, index, max_trap_days, r, row, df_list, df, use)
   
  #Rename dataframe:
   df <- df_2 ; rm(df_2, sum_before, sum_after)
  
  #Add total number of individuals caught per day in all open traps:
   df <- df %>%
     rowwise() %>%
     mutate(Total_Art = sum(Plot_A, Plot_B, Plot_C, Plot_D, Plot_E, Plot_F, Plot_G, Plot_H, na.rm=TRUE))
   
  #Specify per day the number of open traps where arthropods were counted:
   df <- df %>%
     rowwise() %>%
     mutate(Total_Traps = sum(!is.na(Plot_A), !is.na(Plot_B), !is.na(Plot_C), !is.na(Plot_D),
                              !is.na(Plot_E), !is.na(Plot_F), !is.na(Plot_G), !is.na(Plot_H)))
   
  #Calculate average number of individuals caught of each taxon per trap per day
   df$Mean_Art <- df$Total_Art / df$Total_Traps
   
  #Add day of year (Julian Day, taking into account leap years):  
   df$doy <- strftime(df$datetime, format = "%j", tz="GMT")
   
  #add year
   df$year <- as.factor(strftime(df$datetime, format = "%Y", tz="GMT"))
   
  #Change NA in the columns Phylum, Order, Family, Genus, Species to 'sp.'
   df$Phylum[is.na(df$Phylum)] <- "sp."
   df$Order[is.na(df$Order)] <- "sp."
   df$Family[is.na(df$Family)] <- "sp."
   df$Genus[is.na(df$Genus)] <- "sp."
   df$Species[is.na(df$Species)] <- "sp."
   
   #Save dataframe:
   write.csv(df, file="Versluijs2022_Output_2-ArthropodDataZAC_DataPreparation_AdjustedDataZAC.csv", row.names = F) 

############################################################################################################################
############################################################################################################################
############################################################################################################################
