
##############################################################################################################################################

#(3): Add biomass to the Zackenberg pitfall data - Case study Zackenberg regressions vs Knipovich regressions

#In this script we add estimates of biomass to the Zackenberg pitfall dataset 1996-2020 using (1) allometric relationships
#from Zackenberg, and (2) allometric relationships from Knipovich. However, as we do not have regressions from both sites 
#for all arthropod families we restrict ourselves to the seven (excluding collembola) families for which we do have data 
#from both sites. In script VI we then use these data to study whether (I) the timing of the yearly arthropod peak at ZAC, 
#and (II) the daily arthropod biomass per sticky trap at ZAC differs when using these biomass estimates calculated
#using regressions from these two different sites.

#In this script, we first create a subset of the Zackenberg pitfall dataset (1996 - 2020) by only selecting those families  
#for which we have a regression from both arctic sites. We then sample lengths of all specimens in this dataframe as these 
#were not measured directly. This approach is conduced by random sampling of lengths for all specimens from the family 
#specific length distribution in an additional dataset. Subsequently, the corresponding biomass of each specimen is 
#calculated separately for TAY and ZAC using our family specific length-biomass regressions. This thus results in two 
#different biomass columns in the dataframe.

#Overview:
#Line 0033 - 0049: Initialize workspace and load auxiliary functions
#Line 0050 - 0143: Read dataframes
#Line 0144 - 0239: Adjust dataframes
#Line 0240 - 0259: Data checks
#Line 0260 - 0917: Add lengths and biomass estimates for each specimen in 'df'
#Line 0918 - 0957: Data checks
#Line 0958 - 0992: Data adjustments
#Line 0993 - 1007: Save Output

#Contact: tom.versluijs@gmail.com
#Date: 2022/11/01

############################################################################################################################################### 

#(I): Initialize workspace and load auxiliary functions

###############################################################################################################################################

 #Clear Workspace
  rm(list=ls())
 
 #Specify Auxiliary functions
 
  #Predict biomass from 'length_mm' as specified in a 'newdata' dataframe using the function 'mod_func' with parameters B0, B1 and SF:
   predict_tom <- function(newdata, mod_func, B0, B1, SF, ...) {
    tmp <- mod_func(newdata[,"length_mm"], B0=B0, B1=B1, SF=SF)
    return(tmp)
    }
 
############################################################################################################################################### 
 
#(II): Read dataframes
   
###############################################################################################################################################    

  #(II.1): Read dataframe with all Biobasis pitfall samples from ZAC 1996 - 2020
   df <- read.csv("Versluijs2022_Output_2-ArthropodDataZAC_DataPreparation_AdjustedDataZAC.csv", header=T)

   #METADATA for 'df':
    #Date = Date of sampling
    #Hour = Start hour of sampling (UTC)
    #PlotID = Plot ID
    #Sorter = Name of labworker(s)
    #Days A = Number of days the traps have been open in section A, or (in 1996) sections A-D combined. For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
    #Days B = Number of days the traps have been open in section B, or (in 1996) sections A-D combined. For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
    #Days C = Number of days the traps have been open in section C, or (in 1996) sections A-D combined. For Art1, C refers to the East chamber (2006-13).
    #Days D = Number of days the traps have been open in section D, or (in 1996) sections A-D combined. For Art1, D refers to the West chamber (2006-13).
    #Days E = Number of days the traps have been open in section E.
    #Days F = Number of days the traps have been open in section F.
    #Days G = Number of days the traps have been open in section G.
    #Days H = Number of days the traps have been open in section H.
    #Phylum = Phylum of the collected arthropod
    #Order = Order of the collected arthropod
    #Family = Family or super-family of the collected arthropod
    #Genus = Genus of the collected arthropod
    #Species = Species of the collected arthropod
    #A = The number of collected individuals from section A, or (in 1996) sections A-D combined.  For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
    #B = The number of collected individuals from section B, or (in 1996) sections A-D combined.  For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
    #C = The number of collected individuals from section C, or (in 1996) sections A-D combined.  For Art1, C refers to the East chamber (2006-13).
    #D = The number of collected individuals from section D, or (in 1996) sections A-D combined.  For Art1, D refers to the West chamber (2006-13).
    #E = The number of collected individuals from section E.
    #F = The number of collected individuals from section F.
    #G = The number of collected individuals from section G.
    #H = The number of collected individuals from section H.
    #Field_remarks = Remarks from the field observer
    #Sorting_remarks = Remarks from the sorter
    #General_remarks = General remarks 
   
  #(II.2): Read dataframe with length biomass equations based on ZAC 2015 pitfall data
   df_LB <- read.csv(file="Versluijs2022_Output_1-LengthBiomassRegressions_TableS1.csv", header=T)
   colnames(df_LB) <- c("taxon", "N", "min_mm", "max_mm", "B0", "B1", "SF", "Weight_level", "Model_mm_mg", "Location")
 
   #METADATA for 'df_LB':
    #taxon = taxonomic group
    #N = sample size for fitting of allometric model
    #min_mm = size of smallest individual of this taxon when fitting of allometric model [mm]
    #max_mm = size of largest individual of this taxon when fitting of allometric model [mm]
    #B0 = B0 parameter of allometric model
    #B1 = B1 parameter of allometric model
    #SF = Smearing factor parameter of allometric model
    #Weight_level = indicating whether weight measurements are individual measurements (individual) or averages per length group (average)
    #Model_mm_mg = Formula of best supported allometric model of relationship between bodylength [mm] and drymass [mg]
    #Location = Location where arthropods were collected (Zackenberg - ZAC, or Knipovich - TAY)
   
  #(II.3): Read and combine dataframes from which lengths will be sampled:
   
    #Read dataframe with all ZAC15 pitfall data
     df_pt <- read.csv("Versluijs2022_Input_3-ArthropodDataZAC_AddBiomass_PitfallZAC15.csv", header=T)
     df_pt <- df_pt[,c("order", "taxon", "length_mm", "weight_mg")]
     colnames(df_pt) <- c("Order", "Family", "length_mm", "weight_mg")
 
    #Read dataframe with the SELECTION of TAY data for families that were not present in the Zackenberg 2015 pitfall data (e.g. Tipulidae, Coleoptera)
     df_pt2 <- read.csv("Versluijs2022_Input_4-ArthropodDataZAC_AddBiomass_PitfallTAY18.csv", header=T)
     df_pt2 <- df_pt2[,c("order", "taxon", "length_mm", "weight_mg")]
     colnames(df_pt2) <- c("Order", "Family", "length_mm", "weight_mg")
 
    #Merge ZAC and TAY datasets
     df_pt <- rbind(df_pt, df_pt2)  ;  rm(df_pt2)
 
    #Read dataframe with sticky trap data from Zackenberg for 2019 and 2021
     df_st <- read.csv("Versluijs2022_Input_5-ArthropodDataZAC_AddBiomass_StickyTrapZAC19-21.csv", header=T)
     df_st <- df_st[,c("Order", "Family", "length_mm")]
     df_st$weight_mg <- NA
     colnames(df_st) <- c("Order", "Family", "length_mm", "weight_mg")
 
    #Combine pitfall and sticky trap dataframes with measured lengths per taxon
     df_lengths <- rbind(df_pt, df_st)
     rm(df_st, df_pt)
     #length(df_lengths$length_mm[!is.na(df_lengths$length_mm)]) #42,295 individuals

     #'df_lengths' contains length measures for 42,295 individual specimens. We will sample lengths from this dataframe to add them to the
     #Zackenberg pitfall dataset 'df' which only contains counts of identified individuals of which lengths and weights were not measured.
     
     #METADATA for 'df_lengths':
      #Order = taxonomic order
      #Family = taxonomic family
      #length_mm = bodylength of specimen [mm]
      #weight_mg = weight of specimen [mg]
      #taxon = taxonomic group [Order_Family]
     
   #(II.4): Specify for which families we have a regression from both TAY and ZAC (note that we exclude Collembola in the next script):
     families_selection <- c("Chironomidae", "Empididae", "Ichneumonidae", "Linyphiidae", "Muscidae", "Mycetophilidae", "Sciaridae", "Collembola")
     
#######################################################################################################################################################   
 
#(III): Adjust dataframes
     
#######################################################################################################################################################   

  #III.1: df_lengths:
     
   #Remove rows where order is NA in df_lengths
    df_lengths <- df_lengths[!is.na(df_lengths$Order),]
   
   #Remove rows where length_mm is NA
    df_lengths <- df_lengths[!is.na(df_lengths$length_mm),]
 
   #Clean up several naming issues in df_lengths for consistency with other dataframes
    sort(unique(df_lengths$Family))
    df_lengths[df_lengths$Family=="", "Family"] <- "sp."
    df_lengths[df_lengths$Family=="Bombus sp.", "Family"] <- "Apidae"
    df_lengths[df_lengths$Family=="Larvae Noctuidae", "Family"] <- "Noctuidae larvae"
    df_lengths[df_lengths$Family=="Lycosidae Egg Sack", "Family"] <- "Lycosidae eggsac"
    df_lengths[df_lengths$Family=="Lycosidae Eggsac", "Family"] <- "Lycosidae eggsac"
    df_lengths[df_lengths$Family=="Lycosidae Eggsack", "Family"] <- "Lycosidae eggsac"
    df_lengths[df_lengths$Family=="Mycetophiliidae", "Family"] <- "Mycetophilidae"
    df_lengths[df_lengths$Family=="Scatophagidae", "Family"] <- "Scathophagidae"
    df_lengths[df_lengths$Family=="v", "Family"] <- "sp."
 
   #Create a taxon column
    df_lengths$taxon <- paste0(df_lengths$Order, "_", df_lengths$Family)
    
   #Restrict to subset of families in families_selection
    df_lengths <- df_lengths[df_lengths$Family %in% families_selection,]
    
  #III.2: df
    
   #Clean up several naming issues in df for consistency with other dataframes
    sort(unique(df$Family))
    df[df$Phylum=="Tardigrada", "Order"] <- "Tardigrade sp."
    df[df$Phylum=="Nematoda", "Order"] <- "Nematoda sp."
    df[df$Family== "Diptera larvae", "Family"] <- "Larvae sp."
    df[df$Family== "Hymenoptera larvae", "Family"] <- "Larvae sp."
    df[df$Family== "Lepidoptera larvae", "Family"] <- "Larvae sp."
    df[df$Family== "unidentified", "Family"] <- "sp."
    df[df$Order=="Aranea", "Order"] <- "Araneae"
    df[grepl("Pieri", df$Family), "Family"] <- "Pieridae"
    df[df$Order=="Enchytraeida", "Order"] <- "Haplotaxida"
    df[grepl("Triopsidae", df$Family), "Family"] <- "Triopsidae"
 
   #Create a taxon column
    df$taxon <- paste0(df$Order, "_", df$Family)
    
   #Add rownumbers to df to be able to retrace original rows of the dataframe
    df$ID <- 1:nrow(df)
    
   #Retain only those rows in df where Total_Art > 0
    df_full <- df #backup of full dataframe
    df <- df[df$Total_Art > 0.0000001, ]
    df <- droplevels(df)
    
   #Restrict to subset of families in families_selection
    df <- df[df$Family %in% families_selection,]  
    
  #III.3: df_LB
    df_LB$B0 <- sub(" .*", "", df_LB$B0)
    df_LB$B1 <- sub(" .*", "", df_LB$B1)
    df_LB$SF <- sub(" .*", "", df_LB$SF)
    df_LB$B0 <- as.numeric(df_LB$B0) #warnings are ok
    df_LB$B1 <- as.numeric(df_LB$B1) #warnings are ok
    df_LB$SF <- as.numeric(df_LB$SF) #warnings are ok
    
    #Split abbreviated taxon column into an Order and Family column
    df_LB$Order <- NA
    df_LB$Family <- NA
    
      #Order
      df_LB[grepl("Acar", df_LB$taxon), "Order"] <- "Acari"
      df_LB[grepl("Aran", df_LB$taxon), "Order"] <- "Araneae"
      df_LB[grepl("Cole", df_LB$taxon), "Order"] <- "Coleoptera"
      df_LB[grepl("Coll", df_LB$taxon), "Order"] <- "Collembola"
      df_LB[grepl("Dipt", df_LB$taxon), "Order"] <- "Diptera"
      df_LB[grepl("Hyme", df_LB$taxon), "Order"] <- "Hymenoptera"
      df_LB[grepl("Lepi", df_LB$taxon), "Order"] <- "Lepidoptera"
    
      #Family
      df_LB$Family <- sub(".*. ", "", df_LB$taxon)

    #Overwrite the abbreviated taxon column
    df_LB$taxon <- paste0(df_LB$Order, "_", df_LB$Family)  
    
    #Restrict to subset of families in families_selection
    df_LB <- df_LB[df_LB$Family %in% families_selection,]  
    
    #Split dataframe in a ZAC dataframe and a TAY dataframe
    df_LB_ZAC <- df_LB[df_LB$Location=="ZAC",]
    df_LB_TAY <- df_LB[df_LB$Location=="TAY",]
    rm(df_LB)

###############################################################################################################################################
    
#(IV): Data checks
    
###############################################################################################################################################
    
   #Create an overview of all taxa for which we have length measurements
    taxon_lengths <- sort(unique(df_lengths$taxon)) 
   
   #Create an overview of all taxa in 'df' for which we do not have any corresponding length measurements  
    taxon_lengths_nomatch <- sort(unique(df$taxon)[!unique(df$taxon) %in% taxon_lengths])
    
   #Create an overview of all taxa for which we have length-biomass equations (or at least some weight measurements where the model formula=NA)
    taxon_LB <- sort(unique(df_LB_ZAC$taxon))
  
   #Create an overview of all taxa in 'df' for which we do not have any corresponding length-biomass equations
    taxon_LB_nomatch <- sort(unique(df$taxon)[!unique(df$taxon) %in% taxon_LB])    
     
    #Note: because we restricted ourselves to the eight taxa in families_selection, there is no taxa for which lenght-measures of L-B regressions are missing
 
###############################################################################################################################################
    
#(V): Add lengths and biomass estimates for each specimen in 'df'
    
###############################################################################################################################################
    
   #We will now run the following loop:
    # (I):   Loop through all rows of 'df'.
    # (II):  select the corresponding taxon (taxon) and total number of individuals caught (n) for each row.
    # (III): LENGTHS: randomly sample n-lengths for this taxon from df_lengths.
    # (IV):  WEIGHTS: Based on the sampled lengths and known length-biomass regressions (df_LB_ZAC and df_LB_TAY) we can now calculate the total biomass 
    #        for each row in 'df'. We do this twice. First based on the ZAC regressions and then based on the TAY regressions.

    #Set seed to get reproducible results
     set.seed(23)

    #Create a count variable to print the progress of the loop below
     count=0
   
    #create empty columns in 'df' to indicate whether the length and biomass estimates are based on sampling, or the use of L-B equations, and whether
    #these calculations were based on the original taxon, or by using a morphologically similar taxon.
     df$length_type <- NA
     df$length_taxon <- NA
     df$weight_type <- NA
     df$weight_taxon <- NA
    
    #Create empty columns for storing vectors of specimen length, biomass and a check if lengths fall within the length range of the used formula
     df$length_vector <- NA
     df$biomass_vector_ZAC <- NA
     df$biomass_vector_TAY <- NA
     df$length_range_ZAC <- NA
     df$length_range_TAY <- NA 
     
    #Create empty columns to store the total biomass in mg per row of 'df' for ZAC and for TAY
     df$Total_biomass_mg_ZAC <- NA
     df$Total_biomass_mg_TAY <- NA
     
    #For debugging
     options(warn=2)
     
    #Loop through all rows of df, sample lengths for each taxon and calculate total biomass
     for(i in 1:nrow(df)){

       #For debugging
       #i=125362 #1, 13, 122, 150, 144693, 171, 84420
       
      #(I): Select current taxon
       taxon <- df[i, 'taxon']

      #(II): Select how many individuals of this taxon were in total caught at this day
       n <- df[i, 'Total_Art']

      #(III): Sample lengths for all 'n' individuals of this taxon

        #Set taxon2 to taxon
         taxon2 <- taxon
           
        #Indicate that length sampling is based on an exact match with taxon
         df$length_type[i] <- "taxon_sample"
           
        #Sample n-times from df_lengths for this taxon
         n_rounded <- ceiling(n)
         lengths_sample_i <- sample(df_lengths[df_lengths$taxon %in% taxon2, "length_mm"], size=n_rounded, replace=TRUE)

        #Store which taxon was used for length sampling
         df$length_taxon[i] <- taxon2

         #lengths_sample_i has a length equal to ceiling(n). That means that even if 0.01 of an individual was caught on this day,    
         #that lengths_sample_i still has length 1. That overestimates the number of length samples. We therefore remove the final 
         #length in lengths_sample_i with a probability of 1 - (n - floor(n)) and store this as length_vector_i.
         length_vector_i <- lengths_sample_i
         if(!n%%1==0){
            
           check_length <- (runif(1, 0, 1) <= (1 -(n - floor(n))))
           if(check_length){length_vector_i <- length_vector_i[-length(length_vector_i)]}
           if(!check_length){length_vector_i <- length_vector_i} 

           }
       
        #store adjusted length vector in 'df$length_vector' as a list
         if(length(length_vector_i) > 0){df$length_vector[i] <- list(length_vector_i)}
         if(!length(length_vector_i) > 0){df$length_vector[i] <- list(NULL)}
         
        #At this point we have thus stored the adjusted length_vector_i in the column df$length_vector. In the code below we continue with length_sample_i
        #(i.e. the original length_vector) to calculate biomass corresponding to each length. We will then adjust the final biomass estimate by multiplying
        #it with (n - floor(n)).

      #(IV): Calculate Total biomass for all individuals with lengths 'lengths_sample_i' of this taxon based on ZAC regressions

         #Indicate that biomass calculations are based on a exact match with taxon ("taxon_equation")
          df$weight_type[i] <- "taxon_equation"

         #Set taxon2 (used for sampling in df_LB_ZAC) to taxon
          taxon2 <- taxon

         #Store which taxon will be used for calculations of biomass
          df$weight_taxon[i] <- taxon2

         #use ZAC L-B equations to estimate Total biomass for the current row in 'df'

           #Extract taxon specific model coefficients and model formula
           mod_B0 <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'B0']
           mod_B1 <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'B1']
           mod_SF <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'SF']
           mod_min_mm <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'min_mm']
           mod_max_mm <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'max_mm']
           mod_form <- df_LB_ZAC[df_LB_ZAC$taxon %in% taxon2, 'Model_mm_mg']

           #In some cases we will have multiple equations for the same taxon. This is most likely because we made separate regressions
           #for e.g. large and small Chironomidae.

           #If only one model formula is available, set this model formula as preferred for all lengths
           if(length(mod_form) == 1){
             mod_form_index <- rep(1, length(lengths_sample_i))
             }

           #If there are multiple model formulas available for a certain taxon, then specify which model is preferred for each length
           #measure by checking whether it falls within the min_mm and max_mm ranges of each model formula.
           if(length(mod_form) > 1){

             #Create an empty indicator to store which model formula is preferred per length measure
             mod_form_index <- rep(NA, length(lengths_sample_i))

             #make sure the formulas are sorted from the formula with smallest length ranges to formula with the largest length ranges
             order_models <- order(mod_min_mm)
             mod_B0 <- mod_B0[order_models]
             mod_B1 <- mod_B1[order_models]
             mod_SF <- mod_SF[order_models]
             mod_min_mm <- mod_min_mm[order_models]
             mod_max_mm <- mod_max_mm[order_models]
             mod_form <- mod_form[order_models]

             #Prefer model formula 1 if the length measure is below the max_mm for model formula 1
             mod_form_index[lengths_sample_i <= mod_max_mm[1]] <- 1

             #Prefer model formula 2 if the length measure is larger than the min_mm for model formula 2
             mod_form_index[lengths_sample_i >= mod_min_mm[2]] <- 2

             #If the length measure falls between the max_mm for model formula 1 and min_mm for model formula 2, prefer the model formula
             #for larger specimens (i.e. model formula 2)
             mod_form_index[lengths_sample_i >= mod_max_mm[1] &  lengths_sample_i <= mod_min_mm[2]] <- 2

           }

           #At this point we thus have an indicator 'mod_form_index' that tells us for each length measure in lengths_sample_i which
           #model formula with corresponding parameter values B0, B1 and SF is preferred.

               #Check whether each specimen's length in 'lengths_sample_i' falls within the min-max length ranges of the preferred L-B model
                length_check_i_vector <- ((lengths_sample_i >= mod_min_mm[mod_form_index]) & (lengths_sample_i <= mod_max_mm[mod_form_index]))

                  #Exclude the last entry of 'length_check_i_vector' if the last length of 'length_vector_i' was also excluded
                  if(length(length_vector_i) != length(length_check_i_vector)){
                    length_check_i_vector <- length_check_i_vector[-length(length_check_i_vector)]
                    }

                  #store adjusted length check vector in 'df$length_range_ZAC[i]' as a list
                  if(length(length_check_i_vector) > 0){df$length_range_ZAC[i] <- list(length_check_i_vector)}
                  if(!length(length_check_i_vector) > 0){df$length_range_ZAC[i] <- list(NULL)}

           #If the same model is preferred for all lengths of this taxon:
           if(length(unique(mod_form_index))==1){

             #Reduce vector to a single model formula
             mod_form <- unique(mod_form[mod_form_index])

             #Create an index to indicate which B0, B1 and SF coefficients are associated with this model
             index_coef <- unique(mod_form_index)

             #Change model formula to an actual equation that R can work with
             if(is.na(mod_form)){mod_func <- NA}
             if(!is.na(mod_form)){
               if(mod_form=="Weight = B0"){mod_func <- function(B0, ...){B0}}
               if(mod_form=="Weight = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, ...){B0 + B1 * length_mm}}
               if(mod_form=="ln(Weight/SF) = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * length_mm)}}
               if(mod_form=="ln(Weight/SF) = B0 + B1 * ln(length)"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * log(length_mm))}}
               }

             #Make biomass predictions for current taxon and lengths

             #If an equation is available, calculate biomass using these L-B regressions:
             if(!is.na(mod_form)){

               #Calculate the corresponding biomass for all length measures
               B0 <- ifelse(mod_B0[index_coef]=="NA", NA, as.numeric(mod_B0[index_coef]))
               B1 <- ifelse(mod_B1[index_coef]=="NA", NA, as.numeric(mod_B1[index_coef]))
               SF <- ifelse(mod_SF[index_coef]=="NA", NA, as.numeric(mod_SF[index_coef]))
               taxon_i_biomass <- mod_func(length_mm=lengths_sample_i, B0=B0, B1=B1, SF=SF)

               #(1): Adjust the length of 'taxon_i_biomass_vector' (but retain 'taxon_i_biomass') to match the length of 'length_vector_i'
                 taxon_i_biomass_vector <- taxon_i_biomass

                 #Exclude last biomass estimate of taxon_i_biomass_vector if the last length in length_vector_i was also excluded
                 if(length(length_vector_i) != length(taxon_i_biomass_vector)){
                   taxon_i_biomass_vector <- taxon_i_biomass_vector[-length(taxon_i_biomass_vector)]
                   }

                 #store the biomass vector in 'df$biomass_vector_ZAC[i]' as a list
                 if(length(taxon_i_biomass_vector) > 0){df$biomass_vector_ZAC[i] <- list(taxon_i_biomass_vector)}
                 if(!length(taxon_i_biomass_vector) > 0){df$biomass_vector_ZAC[i] <- list(NULL)}


               #(2) Calculate sum of total biomass per day and store in df$Total_biomass_mg[i]

                 #We need to adjust the last biomass estimate as 'n' was rounded upwards. I.e. if n=18.4100244, then we have now pretended that
                 #n=19. We need to adjust the last (19th) biomass estimate as it does not correspond to a complete individual, but instead 0.4100244
                 #of an individual. We thus multiply the last biomass estimate with 0.4100244 to end up with the correct total biomass. Note that
                 #we continue with the original biomass vector here (i.e. with taxon_i_biomass instead of taxon_i_biomass_vector).

                 #Adjust final biomass estimate if n is not an integer
                 if(!n%%1==0){
                   taxon_i_biomass[length(taxon_i_biomass)] <- (n - floor(n)) * taxon_i_biomass[length(taxon_i_biomass)]
                   }

                 #Sum all biomass estimates for all lengths in 'lengths_sample_i'
                 taxon_i_biomass_sum <- sum(taxon_i_biomass)

                 #Store total biomass in Total_biomass_mg_ZAC column in df
                 df$Total_biomass_mg_ZAC[i] <- taxon_i_biomass_sum

               }

             #If no L-B equation is available (which should not be possible) print an error
             if(is.na(mod_form)){

               print(paste0("i=", i, "  -  No L-B equation available for taxon ", taxon2))
               break

             }

             }

           #If there are multiple models preferred for the sampled lengths of this taxon:
           if(length(unique(mod_form_index))>1){

             #Store all model formulas as a vector
             mod_form_c <- mod_form[mod_form_index]

             #Create an empty list to store the total biomass sum per model formula
             l_biomass_modform <- list()

             #Create an empty list to store the biomass vector per model formula
             l_biomass_vector_modform <- list()

             #Calculate biomass separately for all lengths belonging to each model formula and store all in one combined list. i.e. let's
             #assume we have a vector of 8 lengths (of eight specimens). First we detected which lengths belong to the first model formula
             #and which to the second (this is stored in 'mod_form_index'). We then start by calculating weights for all lengths belonging
             #to the first model formula. Let's say these are the lengths on position 1, 4, 7 and 8. Then we calculate weights for those
             #lengths using the first model formula and store them in a list at position 1, 4, 7 and 8. Then we calculate weights for the
             #remaining lengths (i.e. on position 2, 3, 5 and 6) using the second model formula and store these weights at position 2,3, 5
             #and 6 in the list. We now have a complete list of weights for all lengths and we can sum the result to get to total biomass.
             for(j in unique(mod_form_index)){

               #create an parameter to indicate which length are associated with the current model formula 'j'
               index_mod <- which(mod_form_index==j)

               #Select the corresponding model formula
               mod_form <- unique(mod_form_c[index_mod])

               #Create an index to indicate which B0, B1 and SF coefficients correspond with the current model formula 'j'
               index_coef <- unique(mod_form_index[index_mod])

               #Change model formula to an actual equation that R can work with
               if(is.na(mod_form)){mod_func <- NA}
               if(!is.na(mod_form)){
                 if(mod_form=="Weight = B0"){mod_func <- function(B0, ...){B0}}
                 if(mod_form=="Weight = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, ...){B0 + B1 * length_mm}}
                 if(mod_form=="ln(Weight/SF) = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * length_mm)}}
                 if(mod_form=="ln(Weight/SF) = B0 + B1 * ln(length)"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * log(length_mm))}}
                 }

               #Make biomass predictions for the subset of lengths belonging to the current model formula of index 'j'

               #If an equation is available calculate biomass using these L-B regressions
               if(!is.na(mod_form)){

                 #Calculate the corresponding biomass for all length measures
                 B0 <- ifelse(mod_B0[index_coef]=="NA", NA, as.numeric(mod_B0[index_coef]))
                 B1 <- ifelse(mod_B1[index_coef]=="NA", NA, as.numeric(mod_B1[index_coef]))
                 SF <- ifelse(mod_SF[index_coef]=="NA", NA, as.numeric(mod_SF[index_coef]))
                 taxon_i_biomass <- mod_func(length_mm=lengths_sample_i[index_mod], B0=B0, B1=B1, SF=SF)

                 #(1): Adjust the length of 'taxon_i_biomass_vector' to match the length of 'length_vector_i'
                 taxon_i_biomass_vector <- taxon_i_biomass

                 #Only correct the actual last estimate (not for every mod_form separately, i.e. in the example above only for position 8)
                 if(n_rounded %in% index_mod){

                   #Exclude last biomass estimate of taxon_i_biomass_vector if the last length in length_vector_i was also excluded
                   if(length(length_vector_i) != length(lengths_sample_i)){
                     taxon_i_biomass_vector <- taxon_i_biomass_vector[-length(taxon_i_biomass_vector)]
                     }

                 }

                 #(2): Adjust the final biomass estimate for a non-integer number of individuals

                 #We need to adjust the last biomass estimate as 'n' was rounded upwards. I.e. if n=18.4100244, then we have so far pretended that
                 #n=19. We need to adjust the last (19th) biomass estimate as it does not correspond to a complete individual, but instead 0.4100244
                 #of an individual. We thus multiply the last biomass estimate with 0.4100244 to end up with the correct total biomass. #Note that we
                 #continue with the unadjusted biomass vector here (i.e. with taxon_i_biomass instead of taxon_i_biomass_vector).

                 #Only correct the actual last estimate (not for every mod_form separately)
                 if(n_rounded %in% index_mod){

                   #Only adjust final biomass estimate if n is not an integer
                   if(!n%%1==0){
                     taxon_i_biomass[length(taxon_i_biomass)] <- (n - floor(n)) * taxon_i_biomass[length(taxon_i_biomass)]
                     }

                   }

                 #(3): Calculate the sum of all biomass estimates for the current model formula
                 taxon_i_biomass_sum <- sum(taxon_i_biomass)

                 #(4): Store Total biomass sum for current mod_form in the list 'l_biomass_modform' at position 'j'
                 l_biomass_modform[[which(j==unique(mod_form_index))]] <- taxon_i_biomass_sum

                 #(5): Store biomass vector for current mod_form in the list 'l_biomass_vector_modform' at position 'j'
                 l_biomass_vector_modform[[which(j==unique(mod_form_index))]] <- taxon_i_biomass_vector

                 }

               #If no equation is available (should not be possible) print an error
               if(is.na(mod_form)){

                 #Print error message
                 print(paste0("i=", i, "  -  No L-B equation available for taxon WE DO NEED THIS CODE!!", taxon2))
                 break

               }

              }

             #Combine the biomass estimates and biomass vectors for the different model formulas

             #Check for NA in l_biomass_modform and stop the loop if any are encountered
             if(any(is.na(unlist(l_biomass_modform)))){
               print("Error: NA not allowed in l_biomass_modform")
               break
               }

             #Sum the total biomass for this taxon over both model formulas (which is already corrected for a non-integer number of individuals)
             taxon_i_biomass_sum <- sum(unlist(l_biomass_modform))

             #Combine biomass vectors for both model formulas
             taxon_i_biomass_vector <- unlist(l_biomass_vector_modform)

             #Change order of the combined biomass vector to match the order of length_vector_i

               #Exclude last mod_form_index entry if the last length in length_vector_i was also excluded
               mod_form_index_adj <- mod_form_index
               if(length(length_vector_i) != length(lengths_sample_i)){
                 mod_form_index_adj <- mod_form_index[-length(mod_form_index)]
                 }

               #Change order of biomass vector to match the original order of the length vector
               taxon_i_biomass_vector <- taxon_i_biomass_vector[order(c(which(mod_form_index_adj==unique(mod_form_index_adj)[1]), which(mod_form_index_adj==unique(mod_form_index_adj)[2])))]
               #plot(taxon_i_biomass_vector ~ length_vector_i) #for debugging

             #Store total biomass in 'Total_biomass_mg_ZAC' column in df
             df$Total_biomass_mg_ZAC[i] <- taxon_i_biomass_sum

             #store the biomass vector in 'df$biomass_vector_ZAC[i]' as a list
             if(length(taxon_i_biomass_vector) > 0){df$biomass_vector_ZAC[i] <- list(taxon_i_biomass_vector)}
             if(!length(taxon_i_biomass_vector) > 0){df$biomass_vector_ZAC[i] <- list(NULL)}

             }

      #(V): Calculate Total biomass for all individuals with lengths 'lengths_sample_i' of this taxon based on TAY regressions

         #Indicate that biomass calculations are based on a exact match with taxon ("taxon_equation")
          df$weight_type[i] <- "taxon_equation"

         #Set taxon2 (used for sampling in df_LB_TAY) to taxon
          taxon2 <- taxon

         #Store which taxon will be used for calculations of biomass
          df$weight_taxon[i] <- taxon2

         #use TAY L-B equations to estimate Total biomass for the current row in 'df'

           #Extract taxon specific model coefficients and model formula
           mod_B0 <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'B0']
           mod_B1 <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'B1']
           mod_SF <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'SF']
           mod_min_mm <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'min_mm']
           mod_max_mm <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'max_mm']
           mod_form <- df_LB_TAY[df_LB_TAY$taxon %in% taxon2, 'Model_mm_mg']

           #In some cases we will have multiple equations for the same taxon. This is most likely because we made separate regressions
           #for e.g. large and small Chironomidae.

           #If only one model formula is available, set this model formula as preferred for all lengths
           if(length(mod_form) == 1){
             mod_form_index <- rep(1, length(lengths_sample_i))
             }

           #If there are multiple model formulas available for a certain taxon, then specify which model is preferred for each length
           #measure by checking whether it falls within the min_mm and max_mm ranges of each model formula.
           if(length(mod_form) > 1){

             #Create an empty indicator to store which model formula is preferred per length measure
             mod_form_index <- rep(NA, length(lengths_sample_i))

             #make sure the formulas are sorted from the formula with smallest length ranges to formula with the largest length ranges
             order_models <- order(mod_min_mm)
             mod_B0 <- mod_B0[order_models]
             mod_B1 <- mod_B1[order_models]
             mod_SF <- mod_SF[order_models]
             mod_min_mm <- mod_min_mm[order_models]
             mod_max_mm <- mod_max_mm[order_models]
             mod_form <- mod_form[order_models]

             #Prefer model formula 1 if the length measure is below the max_mm for model formula 1
             mod_form_index[lengths_sample_i <= mod_max_mm[1]] <- 1

             #Prefer model formula 2 if the length measure is larger than the min_mm for model formula 2
             mod_form_index[lengths_sample_i >= mod_min_mm[2]] <- 2

             #If the length measure falls between the max_mm for model formula 1 and min_mm for model formula 2, prefer the model formula
             #for larger specimens (i.e. model formula 2)
             mod_form_index[lengths_sample_i >= mod_max_mm[1] &  lengths_sample_i <= mod_min_mm[2]] <- 2

           }

           #At this point we thus have an indicator 'mod_form_index' that tells us for each length measure in lengths_sample_i which
           #model formula with corresponding parameter values B0, B1 and SF is preferred.

               #Check whether each specimen's length in 'lengths_sample_i' falls within the min-max length ranges of the preferred L-B model
                length_check_i_vector <- ((lengths_sample_i >= mod_min_mm[mod_form_index]) & (lengths_sample_i <= mod_max_mm[mod_form_index]))

                  #Exclude the last entry of 'length_check_i_vector' if the last length of 'length_vector_i' was also excluded
                  if(length(length_vector_i) != length(length_check_i_vector)){
                    length_check_i_vector <- length_check_i_vector[-length(length_check_i_vector)]
                    }

                  #store adjusted length check vector in 'df$length_range_TAY[i]' as a list
                  if(length(length_check_i_vector) > 0){df$length_range_TAY[i] <- list(length_check_i_vector)}
                  if(!length(length_check_i_vector) > 0){df$length_range_TAY[i] <- list(NULL)}

           #If the same model is preferred for all lengths of this taxon:
           if(length(unique(mod_form_index))==1){

             #Reduce vector to a single model formula
             mod_form <- unique(mod_form[mod_form_index])

             #Create an index to indicate which B0, B1 and SF coefficients are associated with this model
             index_coef <- unique(mod_form_index)

             #Change model formula to an actual equation that R can work with
             if(is.na(mod_form)){mod_func <- NA}
             if(!is.na(mod_form)){
               if(mod_form=="Weight = B0"){mod_func <- function(B0, ...){B0}}
               if(mod_form=="Weight = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, ...){B0 + B1 * length_mm}}
               if(mod_form=="ln(Weight/SF) = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * length_mm)}}
               if(mod_form=="ln(Weight/SF) = B0 + B1 * ln(length)"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * log(length_mm))}}
               }

             #Make biomass predictions for current taxon and lengths

             #If an equation is available, calculate biomass using these L-B regressions:
             if(!is.na(mod_form)){

               #Calculate the corresponding biomass for all length measures
               B0 <- ifelse(mod_B0[index_coef]=="NA", NA, as.numeric(mod_B0[index_coef]))
               B1 <- ifelse(mod_B1[index_coef]=="NA", NA, as.numeric(mod_B1[index_coef]))
               SF <- ifelse(mod_SF[index_coef]=="NA", NA, as.numeric(mod_SF[index_coef]))
               taxon_i_biomass <- mod_func(length_mm=lengths_sample_i, B0=B0, B1=B1, SF=SF)

               #(1): Adjust the length of 'taxon_i_biomass_vector' (but retain 'taxon_i_biomass') to match the length of 'length_vector_i'
                 taxon_i_biomass_vector <- taxon_i_biomass

                 #Exclude last biomass estimate of taxon_i_biomass_vector if the last length in length_vector_i was also excluded
                 if(length(length_vector_i) != length(taxon_i_biomass_vector)){
                   taxon_i_biomass_vector <- taxon_i_biomass_vector[-length(taxon_i_biomass_vector)]
                   }

                 #store the biomass vector in 'df$biomass_vector_TAY[i]' as a list
                 if(length(taxon_i_biomass_vector) > 0){df$biomass_vector_TAY[i] <- list(taxon_i_biomass_vector)}
                 if(!length(taxon_i_biomass_vector) > 0){df$biomass_vector_TAY[i] <- list(NULL)}


               #(2) Calculate sum of total biomass per day and store in df$Total_biomass_mg[i]

                 #We need to adjust the last biomass estimate as 'n' was rounded upwards. I.e. if n=18.4100244, then we have now pretended that
                 #n=19. We need to adjust the last (19th) biomass estimate as it does not correspond to a complete individual, but instead 0.4100244
                 #of an individual. We thus multiply the last biomass estimate with 0.4100244 to end up with the correct total biomass. Note that
                 #we continue with the original biomass vector here (i.e. with taxon_i_biomass instead of taxon_i_biomass_vector).

                 #Adjust final biomass estimate if n is not an integer
                 if(!n%%1==0){
                   taxon_i_biomass[length(taxon_i_biomass)] <- (n - floor(n)) * taxon_i_biomass[length(taxon_i_biomass)]
                   }

                 #Sum all biomass estimates for all lengths in 'lengths_sample_i'
                 taxon_i_biomass_sum <- sum(taxon_i_biomass)

                 #Store total biomass in Total_biomass_mg_TAY column in df
                 df$Total_biomass_mg_TAY[i] <- taxon_i_biomass_sum

               }

             #If no L-B equation is available (which should not be possible) print an error
             if(is.na(mod_form)){

               print(paste0("i=", i, "  -  No L-B equation available for taxon ", taxon2))
               break

             }

             }

           #If there are multiple models preferred for the sampled lengths of this taxon:
           if(length(unique(mod_form_index))>1){

             #Store all model formulas as a vector
             mod_form_c <- mod_form[mod_form_index]

             #Create an empty list to store the total biomass sum per model formula
             l_biomass_modform <- list()

             #Create an empty list to store the biomass vector per model formula
             l_biomass_vector_modform <- list()

             #Calculate biomass separately for all lengths belonging to each model formula and store all in one combined list. i.e. let's
             #assume we have a vector of 8 lengths (of eight specimens). First we detected which lengths belong to the first model formula
             #and which to the second (this is stored in 'mod_form_index'). We then start by calculating weights for all lengths belonging
             #to the first model formula. Let's say these are the lengths on position 1, 4, 7 and 8. Then we calculate weights for those
             #lengths using the first model formula and store them in a list at position 1, 4, 7 and 8. Then we calculate weights for the
             #remaining lengths (i.e. on position 2, 3, 5 and 6) using the second model formula and store these weights at position 2,3, 5
             #and 6 in the list. We now have a complete list of weights for all lengths and we can sum the result to get to total biomass.
             for(j in unique(mod_form_index)){

               #create an parameter to indicate which length are associated with the current model formula 'j'
               index_mod <- which(mod_form_index==j)

               #Select the corresponding model formula
               mod_form <- unique(mod_form_c[index_mod])

               #Create an index to indicate which B0, B1 and SF coefficients correspond with the current model formula 'j'
               index_coef <- unique(mod_form_index[index_mod])

               #Change model formula to an actual equation that R can work with
               if(is.na(mod_form)){mod_func <- NA}
               if(!is.na(mod_form)){
                 if(mod_form=="Weight = B0"){mod_func <- function(B0, ...){B0}}
                 if(mod_form=="Weight = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, ...){B0 + B1 * length_mm}}
                 if(mod_form=="ln(Weight/SF) = B0 + B1 * length"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * length_mm)}}
                 if(mod_form=="ln(Weight/SF) = B0 + B1 * ln(length)"){mod_func <- function(length_mm, B0, B1, SF, ...){SF * exp(B0 + B1 * log(length_mm))}}
                 }

               #Make biomass predictions for the subset of lengths belonging to the current model formula of index 'j'

               #If an equation is available calculate biomass using these L-B regressions
               if(!is.na(mod_form)){

                 #Calculate the corresponding biomass for all length measures
                 B0 <- ifelse(mod_B0[index_coef]=="NA", NA, as.numeric(mod_B0[index_coef]))
                 B1 <- ifelse(mod_B1[index_coef]=="NA", NA, as.numeric(mod_B1[index_coef]))
                 SF <- ifelse(mod_SF[index_coef]=="NA", NA, as.numeric(mod_SF[index_coef]))
                 taxon_i_biomass <- mod_func(length_mm=lengths_sample_i[index_mod], B0=B0, B1=B1, SF=SF)

                 #(1): Adjust the length of 'taxon_i_biomass_vector' to match the length of 'length_vector_i'
                 taxon_i_biomass_vector <- taxon_i_biomass

                 #Only correct the actual last estimate (not for every mod_form separately, i.e. in the example above only for position 8)
                 if(n_rounded %in% index_mod){

                   #Exclude last biomass estimate of taxon_i_biomass_vector if the last length in length_vector_i was also excluded
                   if(length(length_vector_i) != length(lengths_sample_i)){
                     taxon_i_biomass_vector <- taxon_i_biomass_vector[-length(taxon_i_biomass_vector)]
                     }

                 }

                 #(2): Adjust the final biomass estimate for a non-integer number of individuals

                 #We need to adjust the last biomass estimate as 'n' was rounded upwards. I.e. if n=18.4100244, then we have so far pretended that
                 #n=19. We need to adjust the last (19th) biomass estimate as it does not correspond to a complete individual, but instead 0.4100244
                 #of an individual. We thus multiply the last biomass estimate with 0.4100244 to end up with the correct total biomass. #Note that we
                 #continue with the unadjusted biomass vector here (i.e. with taxon_i_biomass instead of taxon_i_biomass_vector).

                 #Only correct the actual last estimate (not for every mod_form separately)
                 if(n_rounded %in% index_mod){

                   #Only adjust final biomass estimate if n is not an integer
                   if(!n%%1==0){
                     taxon_i_biomass[length(taxon_i_biomass)] <- (n - floor(n)) * taxon_i_biomass[length(taxon_i_biomass)]
                     }

                   }

                 #(3): Calculate the sum of all biomass estimates for the current model formula
                 taxon_i_biomass_sum <- sum(taxon_i_biomass)

                 #(4): Store Total biomass sum for current mod_form in the list 'l_biomass_modform' at position 'j'
                 l_biomass_modform[[which(j==unique(mod_form_index))]] <- taxon_i_biomass_sum

                 #(5): Store biomass vector for current mod_form in the list 'l_biomass_vector_modform' at position 'j'
                 l_biomass_vector_modform[[which(j==unique(mod_form_index))]] <- taxon_i_biomass_vector

                 }

               #If no equation is available (should not be possible) print an error
               if(is.na(mod_form)){

                 #Print error message
                 print(paste0("i=", i, "  -  No L-B equation available for taxon WE DO NEED THIS CODE!!", taxon2))
                 break

               }

              }

             #Combine the biomass estimates and biomass vectors for the different model formulas

             #Check for NA in l_biomass_modform and stop the loop if any are encountered
             if(any(is.na(unlist(l_biomass_modform)))){
               print("Error: NA not allowed in l_biomass_modform")
               break
               }

             #Sum the total biomass for this taxon over both model formulas (which is already corrected for a non-integer number of individuals)
             taxon_i_biomass_sum <- sum(unlist(l_biomass_modform))

             #Combine biomass vectors for both model formulas
             taxon_i_biomass_vector <- unlist(l_biomass_vector_modform)

             #Change order of the combined biomass vector to match the order of length_vector_i

               #Exclude last mod_form_index entry if the last length in length_vector_i was also excluded
               mod_form_index_adj <- mod_form_index
               if(length(length_vector_i) != length(lengths_sample_i)){
                 mod_form_index_adj <- mod_form_index[-length(mod_form_index)]
                 }

               #Change order of biomass vector to match the original order of the length vector
               taxon_i_biomass_vector <- taxon_i_biomass_vector[order(c(which(mod_form_index_adj==unique(mod_form_index_adj)[1]), which(mod_form_index_adj==unique(mod_form_index_adj)[2])))]
               #plot(taxon_i_biomass_vector ~ length_vector_i) #for debugging

             #Store total biomass in 'Total_biomass_mg_TAY' column in df
             df$Total_biomass_mg_TAY[i] <- taxon_i_biomass_sum

             #store the biomass vector in 'df$biomass_vector_TAY[i]' as a list
             if(length(taxon_i_biomass_vector) > 0){df$biomass_vector_TAY[i] <- list(taxon_i_biomass_vector)}
             if(!length(taxon_i_biomass_vector) > 0){df$biomass_vector_TAY[i] <- list(NULL)}

             }

      #(VI): print progress every 1000 df-rows:
       count <- count + 1
       if(count==1000){
         count <- 0
         print(paste0(round(100*i/nrow(df), 2), " %"))
         }
     
     }
   
###############################################################################################################################################
     
#(VI): Data checks
     
###############################################################################################################################################
     
    #Check that each row has a weight estimate (Output should be TRUE)
     !any(is.na(df$Total_biomass_mg_ZAC))
     !any(is.na(df$Total_biomass_mg_TAY))
     
    #Check if the length of the length vector, the length of the biomass vector and the length of the length_range vector are the same for all rows in df
     
      #Calculate the length of each vector
      list_length1 <- unlist(lapply(df$length_vector, function(x) length(x)))
      list_length2a <- unlist(lapply(df$biomass_vector_ZAC, function(x) length(x)))
      list_length2b <- unlist(lapply(df$biomass_vector_TAY, function(x) length(x)))
      list_length3a <- unlist(lapply(df$length_range_ZAC, function(x) length(x)))
      list_length3b <- unlist(lapply(df$length_range_TAY, function(x) length(x)))
     
      #Check if all vectors are of equal length (Output should be TRUE)
      tmp <- data.frame(a=list_length1, b=list_length2a, c=list_length3a) #ZAC
      !any(c(any(tmp$a != tmp$b), any(tmp$b != tmp$c), any(tmp$a != tmp$c)))
      tmp <- data.frame(a=list_length1, b=list_length2b, c=list_length3b) #TAY
      !any(c(any(tmp$a != tmp$b), any(tmp$b != tmp$c), any(tmp$a != tmp$c)))
     
    #Conduct a visual check of the length and biomass vector per taxon (for debugging):
      # taxa_tmp <- sort(unique(df$weight_taxon))
      # for(i in taxa_tmp){
      #   tmp <- df[df$weight_taxon==i,]
      #   tmp_ZAC <- data.frame(length=unlist(tmp$length_vector), weight=unlist(tmp$biomass_vector_ZAC))
      #   tmp_TAY <- data.frame(length=unlist(tmp$length_vector), weight=unlist(tmp$biomass_vector_TAY))
      #   tmp_ZAC <- tmp_ZAC[!is.null(tmp_ZAC$length),]
      #   tmp_TAY <- tmp_TAY[!is.null(tmp_TAY$length),]
      #   print(i)
      #   if(nrow(tmp)>0){
      #     with(tmp_ZAC, plot(weight~length, main=i, col="orange"))
      #     with(tmp_TAY, points(weight~length, main=i, col="blue"))
      #     }
      #   }
      
###############################################################################################################################################
      
#(VII): Data adjustments
      
###############################################################################################################################################
      
  #In some cases (especially with linear length-weight formulas) estimated weights can be negative. We set all those cases to zero 
   df$Total_biomass_mg_ZAC[df$Total_biomass_mg_ZAC<0] <- 0
   df$Total_biomass_mg_TAY[df$Total_biomass_mg_TAY<0] <- 0
     
  # #Join 'df' with the original dataframe 'df_full'
  # df_full <- dplyr::left_join(df_full, df[,c("ID", "length_type", "length_taxon", "weight_type", "weight_taxon", "length_vector", "length_range_ZAC", "length_range_TAY", "biomass_vector_ZAC", "biomass_vector_TAY", "Total_biomass_mg_ZAC", "Total_biomass_mg_TAY")], by="ID")
  # df_full$Total_biomass_mg_ZAC[is.na(df_full$Total_biomass_mg_ZAC)] <- 0
  # df_full$Total_biomass_mg_TAY[is.na(df_full$Total_biomass_mg_TAY)] <- 0
    
  #Change length_vector to NA if equal to NULL
  index <- sapply(df$length_vector, is.null)
  df[which(index), "length_vector"] <- NA

  #Change biomass_vector to NA if equal to NULL
  index <- sapply(df$biomass_vector_ZAC, is.null)
  df[which(index), "biomass_vector_ZAC"] <- NA
  index <- sapply(df$biomass_vector_TAY, is.null)
  df[which(index), "biomass_vector_TAY"] <- NA
    
  #Change length_range to NA if equal to NULL
  index <- sapply(df$length_range_ZAC, is.null)
  df[which(index), "length_range_ZAC"] <- NA
  index <- sapply(df$length_range_TAY, is.null)
  df[which(index), "length_range_TAY"] <- NA
    
  #Add mean biomass per trap per day for ZAC and TAY
  df$Mean_biomass_ZAC <- df$Total_biomass_mg_ZAC / df$Total_Traps
  df$Mean_biomass_TAY <- df$Total_biomass_mg_TAY / df$Total_Traps
  
###############################################################################################################################################
  
#(VIII): Save output: 
  
###############################################################################################################################################
  
  #Save dataframe without the five columns containing lists of vectors:
  write.csv(df[,-c(which(colnames(df) %in% c("length_vector", "biomass_vector_ZAC", "biomass_vector_TAY", "length_range_ZAC", "length_range_TAY")))], file="Versluijs2022_Output_3-ArthropodDataZAC_AddBiomass_ZACvsTAY_AdjustedDataZAC_WithBiomass.csv", row.names=F)
    
  #Save df object with the five columns containing lists of vectors:
  saveRDS(df, file = "Versluijs2022_Output_3-ArthropodDataZAC_AddBiomass_ZACvsTAY_AdjustedDataZAC_WithBiomassAndVectors.RDS") 

###############################################################################################################################################################    
###############################################################################################################################################################    
###############################################################################################################################################################    
