
##############################################################################################################################################

#(4): Case study Zackenberg regressions vs Knipovich regressions

#In this script we analyze whether (I) the timing of the yearly arthropod peak at ZAC, and (II) the daily arthropod biomass per 
#sticky trap at ZAC differs when we use the regressions for Zackenberg versus those for Knipovich to calculate these metrics. As we do not 
#have regressions from both sites for all arthropod families we restrict ourselves to the seven (excluding collembola) families for which 
#we do have data from both sites.

 #Overview:
  #Line 0020 - 0123: Initialize workspace and load data
  #Line 0124 - 0375: Calculate statistics of interest for yearly arthropod peak based on ZAC and KNP regressions
  #Line 0376 - 0512: Calculate several statistics to quantify the difference between using ZAC and KNP regressions
  #Line 0513 - 0632: Calculate some summary statistics for the dataset

 #Contact: tom.versluijs@gmail.com
 #Date: 2022/11/01

###################################################################################################################################################################  

#(I): Initialize workspace and load data

###################################################################################################################################################################  

 #Clear workspace:
  rm(list=ls())

 #Load packages
  library(ggplot2)

 #(I.1) Load Arthropod data
 
   #Load dataframe with all pitfall data (Art1 - Art7) from ZAC 1996 - 2020 including estimated biomass by Tom: 
    df_Art <- read.csv(file="Versluijs2022_Output_3-ArthropodDataZAC_AddBiomass_ZACvsTAY_AdjustedDataZAC_WithBiomass.csv", header=T)
    df_Art$datetime <- as.POSIXct(df_Art$datetime, tz="GMT", format = "%Y-%m-%d %H:%M:%S")

    #METADATA for 'df_Art':
     #datetime = Date and time of sampling
     #PlotID = Plot ID
     #Days A = Number of days the traps have been open in section A, or (in 1996) sections A-D combined. For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
     #Days B = Number of days the traps have been open in section B, or (in 1996) sections A-D combined. For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
     #Days C = Number of days the traps have been open in section C, or (in 1996) sections A-D combined. For Art1, C refers to the East chamber (2006-13).
     #Days D = Number of days the traps have been open in section D, or (in 1996) sections A-D combined. For Art1, D refers to the West chamber (2006-13).
     #Days E = Number of days the traps have been open in section E.
     #Days F = Number of days the traps have been open in section F.
     #Days G = Number of days the traps have been open in section G.
     #Days H = Number of days the traps have been open in section H.
     #Phylum = Phylum of the collected arthropod
     #Order = Order of the collected arthropod
     #Family = Family or super-family of the collected arthropod
     #Genus = Genus of the collected arthropod
     #Species = Species of the collected arthropod
     #Plot_A = The number of collected individuals from section A, or (in 1996) sections A-D combined.  For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
     #Plot_B = The number of collected individuals from section B, or (in 1996) sections A-D combined.  For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
     #Plot_C = The number of collected individuals from section C, or (in 1996) sections A-D combined.  For Art1, C refers to the East chamber (2006-13).
     #Plot_D = The number of collected individuals from section D, or (in 1996) sections A-D combined.  For Art1, D refers to the West chamber (2006-13).
     #Plot_E = The number of collected individuals from section E.
     #Plot_F = The number of collected individuals from section F.
     #Plot_G = The number of collected individuals from section G.
     #Plot_H = The number of collected individuals from section H.
     #Total_Art = Total number of arthropods caught in all active traps on this day
     #Total_Traps = Total number of arthropod traps that were active this day
     #Mean_Art = Number of arthropods caught per trap on this day
     #doy = day of year
     #year = year
     #taxon = taxonomic group
     #ID = unique identifier for each row
     #length_type = lengths were either sampled for this taxon ("taxon_sample") or for a similar taxon ("similar_sample")
     #length_taxon = which taxon was used for length sampling
     #weight_type = weights were either calculated using an equation, or sampled from raw biomass data for the current taxon or for a similar taxon
     #weight_taxon = which taxon was used for calculation/sampling of biomass
     #Total_biomass_mg_ZAC = Total biomass caught in all active traps on this day calculated using ZAC regressions [mg]
     #Total_biomass_mg_TAY = Total biomass caught in all active traps on this day calculated using KNP regressions [mg]
     #Mean_biomass_ZAC = Mean biomass caught per trap on this day calculated using ZAC regressions [mg/trap]
     #Mean_biomass_TAY = Mean biomass caught per trap on this day calculated using KNP regressions [mg/trap]
    
 #(I.2) Adjust dataframe 'df_Art'   
    
   #Exclude Art1 (window trap) and Art6 (no data for 1999 - 2018)
    #Note that Art6 was only open in 1996, 1997, 1998 and 2019, 2020
    #Note that Art4 only runs until (and including) 2016
    #Note that Art7 only runs from (and including) 1999
    df_Art <- df_Art[!df_Art$Plot.ID %in% c("Art1", "Art6"), ]
    df_Art <- droplevels(df_Art)

 #(I.3) Create dataframe 'df_Art_date' with the mean number and mean biomass for ZAC and TAY of arthropods per day for each year    
    
   #Sum Mean_Art and Mean_Biomass for each unique doy-year combination separately for the different ART(2-7) plots
    df_Art_date <- with(df_Art, aggregate(list(Mean_Art, Mean_biomass_ZAC, Mean_biomass_TAY), by=list(doy, year, Plot.ID), sum))
    colnames(df_Art_date) <- c("doy", "year", "Plot.ID", "Mean_Art", "Mean_biomass_ZAC", "Mean_biomass_TAY")
  
   #Average Mean_Art and Mean_Biomass for each unique doy-year combination over the different Art(2-7) plots
    df_Art_date <- with(df_Art_date, aggregate(list(Mean_Art, Mean_biomass_ZAC, Mean_biomass_TAY), by=list(doy, year), mean))
    colnames(df_Art_date) <- c("doy", "year", "Mean_Art", "Mean_biomass_ZAC", "Mean_biomass_TAY")
  
   #Order df_Art_date by doy and year: 
    df_Art_date <- df_Art_date[order(df_Art_date$year, df_Art_date$doy),]

   #Restrict 'df_Art_date' to a fixed sampling window per year  
  
     #Sampling periods differ among years. To be able to make comparisons we restrict the dataset to a fixed sampling window (doy - doy)
     doy_range <- cbind(with(df_Art_date, aggregate(doy, by=list(year), min)),  with(df_Art_date, aggregate(doy, by=list(year), max))$x)
     colnames(doy_range) <- c("year", "doy_min", "doy_max")
  
     #Even though sampling started c.a. 8 days later in 2018, we did not exclude 2018 from the analysis because we are comparing 
     #yearly average biomass per trap per day between ZAC and TAY regressions. What matters is the comparison within years, not 
     #among years, and this can be made irrespective of a slightly later start of the sampling season for 2018.
     years <- sort(unique(df_Art_date$year))
     
     #Exclude 2020 because sampling started c.a. 2 weeks later this year, which would lead to biased estimates of phenology
     years <- years[!years %in% c("2020")]

     #All remaining years have a sampling window that includes the period from doy 157 to doy 238
     doy_min=157 #152 is first of June. We chose doy 157
     doy_max=238 #243 is 31st of August. We chose doy 238
     #Note: 2015 starts at doy 159, and 2018 at doy 165 
     #We thus miss the first few days of these two seasons, which is OK as arthropod numbers are very small early in the season
  
     #Exclude all data outside that window
     df_Art_date <- df_Art_date[df_Art_date$doy >= doy_min & df_Art_date$doy <= doy_max,]
     df_Art_date <- droplevels(df_Art_date)
  
################################################################################################################################################################
     
#(II): Calculate statistics of interest for yearly arthropod peak based on ZAC and KNP regressions
     
################################################################################################################################################################
     
 #Create empty lists    
  list_plots_date <- list() 
  list_Biomass_ZAC_peak_date_median <- list()
  list_Biomass_ZAC_peak_date_max <- list()
  list_Biomass_ZAC_peak_max <- list()
  list_Biomass_ZAC_peak_width <- list()
  list_Biomass_ZAC_mean <- list()
  list_Biomass_TAY_peak_date_median <- list()
  list_Biomass_TAY_peak_date_max <- list()
  list_Biomass_TAY_peak_max <- list()
  list_Biomass_TAY_peak_width <- list()
  list_Biomass_TAY_mean <- list()
  
 #Analyse arthropod data per year
  for(i in years){
  
   #Store data of this year in a temporary dataframe:  
    df_tmp <- df_Art_date[df_Art_date$year==i,]  

    ###########################################################################################################
      #(I): Arthropod biomass Zackenberg:
    ###########################################################################################################

      #1: Peak median date (at which date were 50% of biomass caught)
       total_sum <- sum(df_tmp$Mean_biomass_ZAC)
       art_50 <- total_sum * 0.5
       art_cum = 0
       for(j in 1:nrow(df_tmp)){

         #caclulate cumulative arthropod biomass up and including to row 'j'
         art_cum <- art_cum + df_tmp$Mean_biomass_ZAC[j]

         if(art_cum>art_50){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_ZAC[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_50 vs the range between art_cum_before and art_cum_after
           ratio <- (art_50 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           doy_interp <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))

           #Sore doy_interp in list
           list_Biomass_ZAC_peak_date_median[[which(i==years)]] <- doy_interp
           break()
         }

       }

      #2: Peak max date (at which date were the maximum number of biomass caught)
       list_Biomass_ZAC_peak_date_max[[which(i==years)]] <- median(df_tmp$doy[df_tmp$Mean_biomass_ZAC==max(df_tmp$Mean_biomass_ZAC)])

       #Random rounding of date of peak biomass if it is not an integer:
        if(as.integer(list_Biomass_ZAC_peak_date_max[[which(i==years)]]) != list_Biomass_ZAC_peak_date_max[[which(i==years)]]){
          rnum <- runif(1,0,1)
          ifelse(rnum>0.5, list_Biomass_ZAC_peak_date_max[[which(i==years)]] <- ceiling(list_Biomass_ZAC_peak_date_max[[which(i==years)]]),
                           list_Biomass_ZAC_peak_date_max[[which(i==years)]] <- floor(list_Biomass_ZAC_peak_date_max[[which(i==years)]]))

          }

      #3: Height of peak (value of max arthropod biomass caught on a day)
       list_Biomass_ZAC_peak_max[[which(i==years)]] <- max(df_tmp$Mean_biomass_ZAC)

      #4: Width of peak (window between date at which 25% and 75% of biomass were caught)
       art_25 <- total_sum * 0.25
       art_75 <- total_sum * 0.75
       art_cum = 0
       for(j in 1:nrow(df_tmp)){
         art_cum <- art_cum + df_tmp$Mean_biomass_ZAC[j]

         if(art_cum>art_25){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_ZAC[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_25 vs the range between art_cum_before and art_cum_after
           ratio <- (art_25 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           window_start_Biomass <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))
           break()
         }
       }
       art_cum = 0
       for(j in 1:nrow(df_tmp)){
         art_cum <- art_cum + df_tmp$Mean_biomass_ZAC[j]

         if(art_cum>art_75){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_ZAC[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_75 vs the range between art_cum_before and art_cum_after
           ratio <- (art_75 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           window_end_Biomass <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))
           break()
         }
       }
       list_Biomass_ZAC_peak_width[[which(i==years)]] <- window_end_Biomass - window_start_Biomass

      #5: Average biomass per trap per day
       list_Biomass_ZAC_mean[[which(i==years)]] <- mean(df_tmp$Mean_biomass_ZAC)

    ###########################################################################################################
      #(II): Arthropod biomass Taymir:
    ###########################################################################################################

      #1: Peak median date (at which date were 50% of biomass caught)
       total_sum <- sum(df_tmp$Mean_biomass_TAY)
       art_50 <- total_sum * 0.5
       art_cum = 0
       for(j in 1:nrow(df_tmp)){

         #caclulate cumulative arthropod biomass up and including to row 'j'
         art_cum <- art_cum + df_tmp$Mean_biomass_TAY[j]

         if(art_cum>art_50){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_TAY[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_50 vs the range between art_cum_before and art_cum_after
           ratio <- (art_50 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           doy_interp <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))

           #Sore doy_interp in list
           list_Biomass_TAY_peak_date_median[[which(i==years)]] <- doy_interp
           break()
         }

       }

      #2: Peak max date (at which date were the maximum number of biomass caught)
       list_Biomass_TAY_peak_date_max[[which(i==years)]] <- median(df_tmp$doy[df_tmp$Mean_biomass_TAY==max(df_tmp$Mean_biomass_TAY)])

       #Random rounding of date of peak biomass if it is not an integer:
        if(as.integer(list_Biomass_TAY_peak_date_max[[which(i==years)]]) != list_Biomass_TAY_peak_date_max[[which(i==years)]]){
          rnum <- runif(1,0,1)
          ifelse(rnum>0.5, list_Biomass_TAY_peak_date_max[[which(i==years)]] <- ceiling(list_Biomass_TAY_peak_date_max[[which(i==years)]]),
                           list_Biomass_TAY_peak_date_max[[which(i==years)]] <- floor(list_Biomass_TAY_peak_date_max[[which(i==years)]]))

          }

      #3: Height of peak (value of max arthropod biomass caught on a day)
       list_Biomass_TAY_peak_max[[which(i==years)]] <- max(df_tmp$Mean_biomass_TAY)

      #4: Width of peak (window between date at which 25% and 75% of biomass were caught)
       art_25 <- total_sum * 0.25
       art_75 <- total_sum * 0.75
       art_cum = 0
       for(j in 1:nrow(df_tmp)){
         art_cum <- art_cum + df_tmp$Mean_biomass_TAY[j]

         if(art_cum>art_25){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_TAY[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_25 vs the range between art_cum_before and art_cum_after
           ratio <- (art_25 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           window_start_Biomass <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))
           break()
         }
       }
       art_cum = 0
       for(j in 1:nrow(df_tmp)){
         art_cum <- art_cum + df_tmp$Mean_biomass_TAY[j]

         if(art_cum>art_75){

           #Calculate cumulative sum before threshold was crossed
           art_cum_before <- art_cum - df_tmp$Mean_biomass_TAY[j]

           #Calculate cumulative sum after threshold was crossed
           art_cum_after <- art_cum

           #Calculate ratio of art_75 vs the range between art_cum_before and art_cum_after
           ratio <- (art_75 - art_cum_before) / (art_cum_after - art_cum_before)

           #Use this ratio to calculate the interpolated day at which the exact threshold was crossed
           window_end_Biomass <- df_tmp$doy[j-1] + (ratio *  (df_tmp$doy[j] - df_tmp$doy[j-1]))
           break()
         }
       }
       list_Biomass_TAY_peak_width[[which(i==years)]] <- window_end_Biomass - window_start_Biomass
    
      #5: Average biomass per trap per day
       list_Biomass_TAY_mean[[which(i==years)]] <- mean(df_tmp$Mean_biomass_TAY)
       
    ###########################################################################################################
     #(III): Store plots for all years: 
    ###########################################################################################################

     #Store plots for all years: 
      list_plots_date[[which(i==years)]] <- ggplot(data=df_Art_date[df_Art_date$year==i,])+
        geom_line(aes(x=doy, y=Mean_biomass_ZAC), col="darkorange", size=0.75, lty=1)+
        geom_line(aes(x=doy, y=Mean_biomass_TAY), col="black", size=0.75, lty=1)+
        geom_vline(xintercept=list_Biomass_ZAC_peak_date_median[[which(i==years)]], col="darkorange", lty=2, lwd=0.5)+ 
        geom_vline(xintercept=list_Biomass_TAY_peak_date_median[[which(i==years)]], col="black", lty=2, lwd=0.5)+ 
        #geom_hline(yintercept=list_Art_peak_max[[which(i==years)]], col="black", lty=1)+
        #geom_hline(yintercept=list_Biomass_ZAC_peak_max[[which(i==years)]], col="black", lty=2)+
        xlim(min(df_Art_date$doy), max(df_Art_date$doy))+
        ylim(0, max(df_Art_date$Mean_biomass_ZAC, df_Art_date$Mean_biomass_TAY))+
        xlab("Day of Year")+
        ylab("Mean biomass caught per day per trap")+
        ggtitle(paste0("Zackenberg ", i))+
        theme_classic()+ 
        theme(plot.title = element_text(hjust=0.5))
  
    }

 #Save plots of arthropod peaks per year:  
  pdf("Versluijs2022_Output_4-Analysis_ZACvsTAY_ArthropodPeaks_Years.pdf")
  list_plots_date
  dev.off()
  
 #Summarize all data into a dataframe:
  df_Art_summary <- do.call(rbind, Map(data.frame, year=years, Biomass_ZAC_peak_date_max=list_Biomass_ZAC_peak_date_max, 
                                       Biomass_ZAC_peak_date_median=list_Biomass_ZAC_peak_date_median, Biomass_ZAC_peak_max=list_Biomass_ZAC_peak_max, 
                                       Biomass_ZAC_peak_width=list_Biomass_ZAC_peak_width, Biomass_ZAC_mean=list_Biomass_ZAC_mean,
                                       Biomass_TAY_peak_date_max=list_Biomass_TAY_peak_date_max, 
                                       Biomass_TAY_peak_date_median=list_Biomass_TAY_peak_date_median, Biomass_TAY_peak_max=list_Biomass_TAY_peak_max, 
                                       Biomass_TAY_peak_width=list_Biomass_TAY_peak_width, Biomass_TAY_mean=list_Biomass_TAY_mean))  

###################################################################################################################################################################  
  
#(III): Calculate several statistics to quantify the difference between using ZAC and KNP regressions
  
###################################################################################################################################################################  
  
 #(III.1): Difference in 50% date based on ZAC vs TAY regressions:

   #Plot average 50% (peak date) based on (I) Zackenberg regressions, and (II) Taymir regressions
    p1 <- ggplot()+
      geom_point(data=df_Art_summary, aes(x=year, y=Biomass_TAY_peak_date_median), pch=16, col="black", size=2)+
      geom_point(data=df_Art_summary, aes(x=year, y=Biomass_ZAC_peak_date_median), pch=16, col="darkorange", size=2)+
      geom_line(data=df_Art_summary, aes(x=year, y=Biomass_TAY_peak_date_median), col="black", size=0.5)+
      geom_line(data=df_Art_summary, aes(x=year, y=Biomass_ZAC_peak_date_median), col="darkorange", size=0.5)+
      theme_classic()+
      xlab("Year")+
      ylab("Date of arthropod biomas peak (doy)")
  
      ggsave("Versluijs2022_Output_4-Analysis_ZACvsTAY_DateBiomassPeak.pdf", plot=p1, width = 6, height = 5)
  
   #Create a boxplot to show the average difference in average biomass per trap per day between ZAC and TAY regressions 
      
     #reshape dataframe
     df_peakdate <- reshape2::melt(df_Art_summary[,c("year", "Biomass_ZAC_peak_date_median", "Biomass_TAY_peak_date_median")], id.vars=c("year"))
     df_peakdate$variable <- as.character(df_peakdate$variable)
     df_peakdate[df_peakdate$variable=="Biomass_ZAC_peak_date_median","variable"] <- "ZAC"
     df_peakdate[df_peakdate$variable=="Biomass_TAY_peak_date_median","variable"] <- "KNP"
     df_peakdate$variable <- as.factor(df_peakdate$variable)
     colnames(df_peakdate) <- c("year", "site", "peakdate")
     df_peakdate$year <- as.numeric(df_peakdate$year)
      
     #Create boxplot:
     p2 <- ggplot()+
      geom_boxplot(data=df_peakdate, aes(x=site, y=peakdate))+
      geom_point(data=df_peakdate, aes(x=site, y=peakdate, color=site), position=position_dodge(width=0.1))+
      scale_color_manual(values = c("KNP"="black", "ZAC"="darkorange"))+
      theme_classic()+
      theme(legend.position="none", axis.title.x=element_blank())+
      ylab("Date of arthropod biomass peak (doy)")
  
    ggsave("Versluijs2022_Output_4-Analysis_ZACvsTAY_DateBiomassPeak_boxplot.pdf", plot=p2, width = 6, height = 5)
  
   #Calculate means and min/max for difference between peaks based on ZAC and TAY regressions:
    df_Art_summary$diff_Biomass_peak_date_median <- df_Art_summary$Biomass_TAY_peak_date_median - df_Art_summary$Biomass_ZAC_peak_date_median
    mean(df_Art_summary$diff_Biomass_peak_date_median)
    min(df_Art_summary$diff_Biomass_peak_date_median)
    max(df_Art_summary$diff_Biomass_peak_date_median)
    
   #Calculate 95% quantile CI using non-parametric (case) bootstrapping:
    
    #Specify the sampling function 
    sampfun_case <- function(data) {
      
      #Re-sample dataframe rows (cases) with replacement
      data_bootcase <- data[sample(length(data), replace=TRUE)]
      return(data_bootcase)
      
    } 
    
    #Conduct hierarchical case bootstrapping by iterating the following process 10000 times: (I) use sampfun_case to generate newly sampled  
    #datasets, (II) Calculate the mean value of each dataset. (III) Store these means in a vector 'y'. (IV) Calculate 95% quantile
    #confidence intervals for this vector 'y'.
    set.seed(23)
    y <- replicate(10000, mean(sampfun_case(data=df_Art_summary$diff_Biomass_peak_date_median)))
    
    #Calculated 95% quantile confidence intervals of the stored means
    boot_confint=0.95
    lwr_ci <- (1-boot_confint)/2
    CI_quantile <- t(quantile(na.omit(y), probs=c(lwr_ci, 1-lwr_ci))) #CI_bca <- coxed::bca(na.omit(y), conf.level=boot_confint)
    CI_quantile
    #Thus, peak in biomass based on ZAC regressions occurs on average 0.13 days (0.03, 0.26) after the peak in biomass based on KNP regressions.
  
 #(III.2): Difference in average biomass per trap per day based on ZAC vs KNP regressions:
  
   #reshape dataframe
    df_avg_biomass <- reshape2::melt(df_Art_summary[,c("year", "Biomass_ZAC_mean", "Biomass_TAY_mean")], id.vars=c("year"))
    df_avg_biomass$variable <- as.character(df_avg_biomass$variable)
    df_avg_biomass[df_avg_biomass$variable=="Biomass_ZAC_mean","variable"] <- "ZAC"
    df_avg_biomass[df_avg_biomass$variable=="Biomass_TAY_mean","variable"] <- "KNP"
    df_avg_biomass$variable <- as.factor(df_avg_biomass$variable)
    colnames(df_avg_biomass) <- c("year", "site", "biomass")
    df_avg_biomass$year <- as.numeric(df_avg_biomass$year)
    
   #Plot average biomass per trap per day based on (I) Zackenberg regressions, and (II) Taymir regressions
    p3 <- ggplot()+
      geom_point(data=df_avg_biomass[df_avg_biomass$site=="KNP",], aes(x=year, y=biomass), pch=16, col="black", size=2)+
      geom_point(data=df_avg_biomass[df_avg_biomass$site=="ZAC",], aes(x=year, y=biomass), pch=16, col="darkorange", size=2)+
      geom_line(data=df_avg_biomass[df_avg_biomass$site=="KNP",], aes(x=year, y=biomass), col="black", size=0.5)+
      geom_line(data=df_avg_biomass[df_avg_biomass$site=="ZAC",], aes(x=year, y=biomass), col="darkorange", size=0.5)+
      theme_classic()+
      xlab("Year")+
      ylab("Average biomass per trap per day (mg)")
      
    ggsave("Versluijs2022_Output_4-Analysis_ZACvsTAY_AverageBiomass_Fig2.pdf", plot=p3, width = 6, height = 5)
   
   #Create a boxplot to show the average difference in average biomass per trap per day between ZAC and TAY regressions 
    p4 <- ggplot()+
     geom_boxplot(data=df_avg_biomass, aes(x=site, y=biomass))+
     geom_point(data=df_avg_biomass, aes(x=site, y=biomass, color=site), position=position_dodge(width=0.1))+
     scale_color_manual(values = c("KNP"="black", "ZAC"="darkorange"))+
     theme_classic()+
     theme(legend.position="none", axis.title.x=element_blank())+
     ylab("Average biomass per trap per day (mg)")
   
    ggsave("Versluijs2022_Output_4-Analysis_ZACvsTAY_AverageBiomass_Fig2_boxplot.pdf", plot=p4, width = 6, height = 5)
   
   #Calculate means and min/max for difference in average biomass per trap per day when using ZAckenberg vs Taymyr regressions:
    df_Art_summary$diff_Biomass_mean <- df_Art_summary$Biomass_TAY_mean / df_Art_summary$Biomass_ZAC_mean
    mean(df_Art_summary$diff_Biomass_mean)
    min(df_Art_summary$diff_Biomass_mean)
    max(df_Art_summary$diff_Biomass_mean)
    
   #Calculate 95% quantile CI using non-parametric (case) bootstrapping:
    
    #Specify the sampling function 
    sampfun_case <- function(data) {
      
      #Re-sample dataframe rows (cases) with replacement
      data_bootcase <- data[sample(length(data), replace=TRUE)]
      return(data_bootcase)
      
    } 
    
    #Conduct hierarchical case bootstrapping by iterating the following process 10000 times: (I) use sampfun_case to generate newly sampled  
    #datasets, (II) Calculate the mean value of each dataset. (III) Store these means in a vector 'y'. (IV) Calculate 95% quantile
    #confidence intervals for this vector 'y'.
    set.seed(23)
    y <- replicate(10000, mean(sampfun_case(data=df_Art_summary$diff_Biomass_mean)))
    
    #Calculated 95% quantile confidence intervals of the stored means
    boot_confint=0.95
    lwr_ci <- (1-boot_confint)/2
    CI_quantile <- t(quantile(na.omit(y), probs=c(lwr_ci, 1-lwr_ci))) #CI_bca <- coxed::bca(na.omit(y), conf.level=boot_confint)
    CI_quantile
    
    #Thus, the average biomass per trap per day based on TAY regressions is on average 23.9% (23.5%, 24.4%) higher than when calculated using ZAC regressions

#####################################################################################################################################################################  
    
#(IV): Calculate some summary statistics for the dataset
    
#####################################################################################################################################################################  
    
 #Clear workspace
  rm(list=ls())
    
 #Load libraries 
  library(ggplot2)
    
 #Read df_full 
  df_full <- readRDS("Versluijs2022_Output_3-ArthropodDataZAC_AddBiomass_ZACvsTAY_AdjustedDataZAC_WithBiomassAndVectors.RDS")
    
  #METADATA for 'df_full':
   #datetime = Date and time of sampling
   #PlotID = Plot ID
   #Days A = Number of days the traps have been open in section A, or (in 1996) sections A-D combined. For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
   #Days B = Number of days the traps have been open in section B, or (in 1996) sections A-D combined. For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
   #Days C = Number of days the traps have been open in section C, or (in 1996) sections A-D combined. For Art1, C refers to the East chamber (2006-13).
   #Days D = Number of days the traps have been open in section D, or (in 1996) sections A-D combined. For Art1, D refers to the West chamber (2006-13).
   #Days E = Number of days the traps have been open in section E.
   #Days F = Number of days the traps have been open in section F.
   #Days G = Number of days the traps have been open in section G.
   #Days H = Number of days the traps have been open in section H.
   #Phylum = Phylum of the collected arthropod
   #Order = Order of the collected arthropod
   #Family = Family or super-family of the collected arthropod
   #Genus = Genus of the collected arthropod
   #Species = Species of the collected arthropod
   #Plot_A = The number of collected individuals from section A, or (in 1996) sections A-D combined.  For Art1, A refers to the North chamber (2006-13). From 2014 onwards, A refers to the North-South chambers combined.
   #Plot_B = The number of collected individuals from section B, or (in 1996) sections A-D combined.  For Art1, B refers to the South chamber (2006-13). From 2014 onwards, B refers to the East-West chambers combined.
   #Plot_C = The number of collected individuals from section C, or (in 1996) sections A-D combined.  For Art1, C refers to the East chamber (2006-13).
   #Plot_D = The number of collected individuals from section D, or (in 1996) sections A-D combined.  For Art1, D refers to the West chamber (2006-13).
   #Plot_E = The number of collected individuals from section E.
   #Plot_F = The number of collected individuals from section F.
   #Plot_G = The number of collected individuals from section G.
   #Plot_H = The number of collected individuals from section H.
   #Total_Art = Total number of arthropods caught in all active traps on this day
   #Total_Traps = Total number of arthropod traps that were active this day
   #Mean_Art = Number of arthropods caught per trap on this day
   #doy = day of year
   #year = year
   #taxon = taxonomic group
   #ID = unique identifier for each row
   #length_type = lengths were either sampled for this taxon ("taxon_sample") or for a similar taxon ("similar_sample")
   #length_taxon = which taxon was used for length sampling
   #weight_type = weights were either calculated using an equation, or sampled from raw biomass data for the current taxon or for a similar taxon
   #weight_taxon = which taxon was used for calculation/sampling of biomass
   #length_vector = vector of lengths for all specimens caught on this day [mm]
   #biomass_vector_ZAC = vector of biomass estimates for all specimens caught on this day based on ZAC regressions [mg]
   #biomass_vector_TAY = vector of biomass estimates for all specimens caught on this day based on KNP regressions [mg]
   #length_range_ZAC = vector to indicate whether each length in 'length_vector' falls within the min-max range of the ZAC allometric regression for this taxon [TRUE/FALSE]
   #length_range_TAY = vector to indicate whether each length in 'length_vector' falls within the min-max range of the KNP allometric regression for this taxon [TRUE/FALSE]
   #Total_biomass_mg_ZAC = Total biomass caught in all active traps on this day calculated using ZAC regressions [mg]
   #Total_biomass_mg_TAY = Total biomass caught in all active traps on this day calculated using KNP regressions [mg]
   #Mean_biomass_ZAC = Mean biomass caught per trap on this day calculated using ZAC regressions [mg/trap]
   #Mean_biomass_TAY = Mean biomass caught per trap on this day calculated using KNP regressions [mg/trap]
  
 #(I): Pre-process df_full
    
   #Restrict to families for which we have a length-biomass regression available for ZAC and TAY:
    families_selection <- c("Chironomidae", "Empididae", "Ichneumonidae", "Linyphiidae", "Muscidae", "Mycetophilidae", "Sciaridae")
    df_full <- df_full[df_full$Family %in% families_selection,]  
    df_full <- droplevels(df_full)
    
   #Exclude Art1 (window trap) and Art6 (no data for 1999 - 2018)
   #Note that Art6 was only open in 1996, 1997, 1998 and 2019, 2020
   #Note that Art4 only runs until (and including) 2016
   #Note that Art7 only runs from (and including) 1999
    df_full <- df_full[!df_full$Plot.ID %in% c("Art1", "Art6"), ]
    df_full <- droplevels(df_full)
    
   #Exclude 2020  (excluding 2018 as well to keep consistent with case study two)
    df_full <- df_full[!df_full$year %in% c("2018", "2020"), ]
    
   #Restrict 'df_Art_date' to a fixed sampling window per year  
    doy_min=157 #152 is first of June. We chose doy 157
    doy_max=238 #243 is 31st of August. We chose doy 238
    df_full <- df_full[df_full$doy >= doy_min & df_full$doy <= doy_max,]
    df_full <- droplevels(df_full)
    
 #(II): Calculate some summarizing statistics 
    
   #Total number of individuals caught
    sum(na.omit(df_full$Total_Art[df_full$year!="2018"])) 
    #Total number of specimens in the final dataset for case-study two is 230,591
    
    #Total number of unique taxonomic groups
    length(na.omit(unique(df_full$taxon[df_full$year!="2018"])))
    #Total number of taxonomic groups analyzed for case study two is 7.

    # #Check for how many specimens the biomass calculations were based on a length the fell outside the min-max range of our equations 
    # tmp <- table(na.omit(unlist(df_full$length_range_ZAC))) 
    # (100 * tmp['FALSE']) / ( tmp['FALSE'] + tmp['TRUE'] )
    # tmp <- table(na.omit(unlist(df_full$length_range_TAY))) 
    # (100 * tmp['FALSE']) / ( tmp['FALSE'] + tmp['TRUE'] )
    # #5.52% falls outside the length range for the ZAC biomass estimates
    # #11.64% falls outside the length range for the ZAC biomass estimates
    # 
    # #Check which taxa were most often extrapolated outside of length ranges 
    # list_length_range_ZAC <- unlist(lapply(df_full$length_range_ZAC, function(x) length(x)))
    # taxon <- rep(df_full$taxon, list_length_range_ZAC)
    # length_range_ZAC <- unlist(df_full$length_range_ZAC)
    # df_length_range_ZAC <- data.frame("taxon"=taxon, "length_range_ZAC"=length_range_ZAC)
    # df_length_range_ZAC <- df_length_range_ZAC[!is.na(df_length_range_ZAC$length_range_ZAC),]
    # df_length_range_ZAC <- df_length_range_ZAC[df_length_range_ZAC$length_range_ZAC==FALSE,]
    # 100 * (table(df_length_range_ZAC$taxon)[c("Diptera_Chironomidae")]) /  sum(sort(table(df_length_range_ZAC$taxon)))
    # #62.55% % of which were due to Chironomidae for ZAC   
    # 
    # #Check which taxa were most often extrapolated outside of length ranges 
    # list_length_range_TAY <- unlist(lapply(df_full$length_range_TAY, function(x) length(x)))
    # taxon <- rep(df_full$taxon, list_length_range_TAY)
    # length_range_TAY <- unlist(df_full$length_range_TAY)
    # df_length_range_TAY <- data.frame("taxon"=taxon, "length_range_TAY"=length_range_TAY)
    # df_length_range_TAY <- df_length_range_TAY[!is.na(df_length_range_TAY$length_range_TAY),]
    # df_length_range_TAY <- df_length_range_TAY[df_length_range_TAY$length_range_TAY==FALSE,]
    # 100 * (table(df_length_range_TAY$taxon)[c("Araneae_Linyphiidae")]) /  sum(sort(table(df_length_range_TAY$taxon)))
    # #49.92% of which were due to Linyphiidae for TAY   

#########################################################################################################################  
#########################################################################################################################  
#########################################################################################################################