
##############################################################################################################################################

#(1): Construct taxon-specific allometric length-biomass relationships based on arthropod pitfall data from Zackenberg 2015 and Knipovich 2018

 #Overview:
   #Line 0017 - 0043: Initialize workspace
   #Line 0044 - 0246: Data preparation
   #Line 0247 - 4679: Construct allometric length-biomass regressions for all arthropod taxa  
   #Line 4680 - 4768: Create a model selection table
   #Line 4769 - 5288: Use case-boostrapping to calculate 95% quantile confidence intervals for model coefficients and model predictions
   #Line 5289 - 5631: Summarize the bootstrapping results and plot the data

 #Contact: tom.versluijs@gmail.com
 #Date: 2022/11/01

####################################################################################################################################

#(I): Initialize workspace

####################################################################################################################################

#Clear Workspace
 rm(list=ls())

#Install packages (should only need to be done once)
 req_packages <- c("dplyr", "MuMIn", "reshape", "ggplot2", "grid", "gridExtra", "coxed", "MASS")
 get_packages <- req_packages[!(req_packages %in% installed.packages()[,"Package"])]
 if(length(get_packages)>0){install.packages(get_packages, dependencies = TRUE)}

#Load packages:
 library(dplyr)
 library(MuMIn)
 library(reshape)
 library(ggplot2)
 library(grid)
 library(gridExtra)
 library(coxed)
 library(MASS)
 library(reshape)

#Load auxilliary functions
 source("Versluijs2022_Script_0-AuxilliaryFunctions.R")

####################################################################################################################################

#(II): Data preparation

####################################################################################################################################

 #Read arthropod data containing length and drymass per specimen
  df <- read.csv("Versluijs2022_Input_1-LengthBiomassRegressions_RawData.csv", header=T)

 #Metadata:
  #-Area: Location where specimens where collected (Knipovich (TAY) or Zackenberg (ZAC))
  #-UTM_x: longitude coordinates of trapping location [UTM].
  #-UTM_y: latitude coordinates of trapping location [UTM].
  #-sample_ID: identifier for trap
  #-trap_type: type of trap (always yellow pitfall traps)
  #-year: trapping year
  #-month: trapping month
  #-day: trapping day
  #-taxon: taxonomic group (generally corresponds to family level)
  #-order: taxomic order
  #-length_measured: length measured under stereomiscroscope
  #-zoom: zoom-level of stereomicroscope 
  #-length_conversion: conversion factor to go from length_measured to length_mm
  #-length_mm: bodylength of specimen [mm]
  #-length_class: grouping factor for individuals of similar length. If weights could not be measured individually then weights are based on
  #              averages for each level of length_class, which are where possible calculated separately for each trap and trapping day.
  #-weight_mg: dryweight of specimen [mg]
  #-weight_level: indicates whether weight measuremens were done on an individual level, or weights are based on averages per length_class
  #-analyzed_at: location and year of lab analysis
  #-identified_by: people/groups responsible for identification of samples
  #-measurements_by: people responsible for measuring and weighing of individuals
  #-Remarks: remarks
 
 #Format dataframe
  df$Area <- as.factor(df$Area)
  df$sample_ID <- as.factor(df$sample_ID)
  df$trap_type <- as.factor(df$trap_type)
  df$year <- as.factor(df$year)
  df$month <- as.factor(df$month)
  df$day <- as.factor(df$day)
  df$taxon <- as.factor(df$taxon)
  df$order <- as.factor(df$order)
  df$length_class <- as.factor(df$length_class)
  df$weight_level <- as.factor(df$weight_level)
  df$analyzed_at <- as.factor(df$analyzed_at)
  df$identified_by <- as.factor(df$identified_by)
  df$measurements_by <- as.factor(df$measurements_by)
  str(df)

 #Remove lycosidae eggsack
  df <- df[!df$taxon=="Lycosidae eggsac",] 
  df <- droplevels(df)
 
 #Inspect how many taxonomic groups are present in the dataset
  taxa_overview <- paste0(df$order, "_", df$taxon)
  table(sort(taxa_overview))
  length(unique(taxa_overview)) #Total number of taxonomic groups identified
  length(unique(taxa_overview[df$Area=="ZAC"])) #Zackenberg
  length(unique(taxa_overview[df$Area=="TAY"])) #Knipovich
  table(df$Area) #Number of specimens identified
  #Thus, total number of specimens identified is 4389 (ZAC=3590, KNP=799) belonging to 42 taxonomic groups (ZAC=34, KNP=19)
 
 #Remove all entries where individuals were not measured:
  df <- df[-which(is.na(df$length_mm)),]
  df <- droplevels(df)
  table(df$Area)
  #Total number of specimens for which length was measured is 4383 (ZAC=3585, KNP=798)

 #There are several weight measurements equal to 0.000. These are either measurement errors, or the scale used was not
 #precise enough. We therefore exclude these measurements from the analysis *Mahrlein 2016).
  df <- df[!df$weight_mg==0 & !is.na(df$weight_mg),]
  df <- droplevels(df)

 #Calculate an average length per length-class:
  #Several small specimens could not be weighed individually. Their weight is thus determined by calculating an average weight of several
  #specimens of a similar length (see length_class column). Where possible, this was done separately for each trap and trapping day (
  #see below for an overview per family). However, as the length of all these specimens is measured individually, each average weight 
  #value thus has multiple entries in our dataframe (i.e. one copy for all the different individual length measures). To reduce 
  #pseudoreplication we reduce these individual length measures to an average length per average weight (i.e. an average length per
  #length class). We conduct these steps below, separately for the Zackenberg and Knipovich dataset.
 
  #(A): Zackenberg
 
   #Select data for Zackenberg
    df_ZAC <- df[df$Area=="ZAC",]
 
   #Inspect the number of specimens per taxa before averaging over length groups
    df_counts_avg_ZAC <- df_ZAC[df_ZAC$weight_level=="average",] %>% group_by(order, taxon) %>% dplyr::summarize(N=n()) 
 
   #Store all data where weights are averages in a separate dataframe:
    df_avg_ZAC <- df_ZAC[df_ZAC$weight_level=="average",]
    
   #Store all data where weights are individual measurements in a separate dataframe
    df_ZAC <- df_ZAC[!df_ZAC$weight_level=="average",]
    #thus, total number of specimens weighed individually for ZAC = 992 (excluding zero weights)
    
   #Calculate an average length per length-class
    
    #For Acari and Collembola we have taken averages per length class (i.e. length-class 1-6, 1-8 respectively). 
    #For Calliphoridae we calculated averages per length class, separately for each sampling day
    #For all other taxons we calculated averages per length_class, separately for each trap and sampling day
    
   #Acari & Collembola  
    df_avg_ZAC1 <- df_avg_ZAC %>%
      filter(order %in% c("Acari", "Collembola")) %>%
      group_by(Area, UTM_x, UTM_y, trap_type, year, weight_level, analyzed_at, identified_by, measurements_by, length_class, order, taxon) %>%
      summarise(length_mm=mean(length_mm), weight_mg=mean(weight_mg))
 
   #Calliphoridae  
    df_avg_ZAC2 <- df_avg_ZAC %>%
      filter(taxon %in% c("Calliphoridae")) %>%
      group_by(Area, UTM_x, UTM_y, trap_type, weight_level, analyzed_at, identified_by, measurements_by, length_class, year, month, day, taxon, order) %>%
      summarise(length_mm=mean(length_mm), weight_mg=mean(weight_mg))
 
   #All others  
    df_avg_ZAC3 <- df_avg_ZAC %>%
      filter(!order %in% c("Acari", "Collembola")) %>%
      filter(!taxon %in% c("Calliphoridae")) %>%
      group_by(Area, UTM_x, UTM_y, trap_type, weight_level, analyzed_at, identified_by, measurements_by, length_class, year, month, day, taxon, order, sample_ID) %>%
      summarise(length_mm=mean(length_mm), weight_mg=mean(weight_mg))
 
   #Combine averages with individual level dataframe  
    df_avg_ZAC <- bind_rows(df_avg_ZAC[c(),], df_avg_ZAC1, df_avg_ZAC2, df_avg_ZAC3)
    df_ZAC <- bind_rows(df_ZAC, df_avg_ZAC)
    rm(df_avg_ZAC, df_avg_ZAC1, df_avg_ZAC2, df_avg_ZAC3) 
 
   #Inspect the number of specimens per taxa after averaging over length-groups
    df_counts_avg_ZAC2 <- df_ZAC[df_ZAC$weight_level=="average",] %>% group_by(order, taxon) %>% dplyr::summarize(N=n()) 
    df_n_avg <- data.frame(area="ZAC", order=df_counts_avg_ZAC$order, taxon=df_counts_avg_ZAC$taxon, n_before=df_counts_avg_ZAC$N, n_after=df_counts_avg_ZAC2$N)
    rm(df_counts_avg_ZAC, df_counts_avg_ZAC2)
    
   #Inspect data on a order level
    ggplot(data=df_ZAC, aes(x=log(length_mm), y=log(weight_mg), fill=order, col=order))+
      geom_point(size=0.8)+
      geom_smooth(method='lm', formula= y~x, se=F, size=1.25)+
      xlab("log length (mm)")+
      ylab("log bodymass (mg)")+
      theme_bw()
    
  #(B): Knipovich
 
   #Select data for Knipovich (TAY)  
    df_TAY <- df[df$Area=="TAY",]
    
   #Inspect the number of specimens per taxa before averaging over length groups
    df_counts_avg_TAY <- df_TAY[df_TAY$weight_level=="average",] %>% group_by(order, taxon) %>% dplyr::summarize(N=n()) 
    
   #Store all datapoints based on averages into a separate dataframe:
    df_avg_TAY <- df_TAY[df_TAY$weight_level=="average",]
    
   #Store the data based on individual weight measurements in a separate dataframe
    df_TAY <- df_TAY[!df_TAY$weight_level=="average",]
    #thus, total number of specimens weighed individually for TAY = 581 (excluding zero weights)
    
   #Calculate an average length per length-class for Collembola

    #Collembola  
     df_avg_TAY1 <- df_avg_TAY %>%
      filter(order %in% c("Collembola")) %>%
      group_by(Area, UTM_x, UTM_y, trap_type, weight_level, analyzed_at, identified_by, measurements_by, length_class, order, taxon) %>%
      summarise(length_mm=mean(length_mm), weight_mg=mean(weight_mg))
    
   #Combine averages with individual level dataframe    
    df_avg_TAY <- bind_rows(df_avg_TAY[c(),], df_avg_TAY1)
    df_TAY <- bind_rows(df_TAY, df_avg_TAY)
    rm(df_avg_TAY, df_avg_TAY1) 
    
   #Inspect the number of specimens per taxa after averaging over length-groups
    df_counts_avg_TAY2 <- df_TAY[df_TAY$weight_level=="average",] %>% group_by(order, taxon) %>% dplyr::summarize(N=n()) 
    df_n_avg2 <- data.frame(area="TAY", order=df_counts_avg_TAY$order, taxon=df_counts_avg_TAY$taxon, n_before=df_counts_avg_TAY$N, n_after=df_counts_avg_TAY2$N)
    df_n_avg <- rbind(df_n_avg, df_n_avg2) ;  rm(df_counts_avg_TAY, df_counts_avg_TAY2, df_n_avg2)
    df_n_avg$name <- paste0(df_n_avg$area, "_", substr(df_n_avg$order, 1, 3), "_" , substr(df_n_avg$taxon, 1, 3), "_ave")

   #Inspect data on a order level
    ggplot(data=df_TAY, aes(x=log(length_mm), y=log(weight_mg), fill=order, col=order))+
     geom_point(size=0.8)+
     geom_smooth(method='lm', formula= y~x, se=F, size=1.25)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_bw()
 
   #Check how many datapoints remain after averaging  
    sum(df_n_avg$n_before)
    sum(df_n_avg$n_after)
    #Thus, total number of specimens before averaging is 2785, while after averaging 188 datapoints (group averages) remain.
    #In total we thus have 992 individual weight measures for ZAC, 581 individual weight measures for TAY and 188 length groups for 
    #which an average weight was calculated. We thus end up with 1761 datapoints (ZAC=1171, KNP=590), excluding zero weight measures.
    
 #Final data preparations:
        
  #Create a name column that is a combination of Area, order, taxon and weight_level
   df_ZAC$name <- paste0(df_ZAC$Area, "_", substr(df_ZAC$order, 1, 3), "_" , substr(df_ZAC$taxon, 1, 3), "_" , substr(df_ZAC$weight_level, 1, 3))
   df_TAY$name <- paste0(df_TAY$Area, "_", substr(df_TAY$order, 1, 3), "_" , substr(df_TAY$taxon, 1, 3), "_" , substr(df_TAY$weight_level, 1, 3))
    
  #Combine dataframes for ZAC and TAY
    
    #keep a backup of the original dataframe
     df_full <- df 
    
    #Combine dataframes and remove separate dataframes for ZAC and KNP
     df <- bind_rows(df_ZAC, df_TAY)
     rm(df_TAY, df_ZAC)

##############################################################################################################################

#(III): Construct allometric length-biomass regressions for all arthropod taxa
 
##############################################################################################################################
 
 #In general we constructed length-biomass regressions at a family level, except for Collembola, Acari and Thysanoptera, for
 #which regression were fitted at sub-class level. We constructed separate regressions for data based on individual weight
 #measurements and data corresponding to average weights per length group.
 
  #Create a variable containing all site specific taxa with sample size > 1
   taxa <- sort(names(table(df$name)[table(df$name) > 1]))
   
  #Create an empty list to store each group's data
   list_taxa <- rep(list(rep(list(), 6)), length(taxa))
    
##############################################################################################################################

  #(1): TAY_Ara_Lin_ind (Knipovich, Araneae, Linyphiidae, individual level data)
   
   #Read data
   i=1
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Thus model m_lm_power is best supported based on AIC, although it does not outperform m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)

###################################################################################################################################
  
  #(2): TAY_Col_Car_ind (Knipovich, Coleoptera, Carabidae, individual level data)
   
   #Read data
   i=2
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.

   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_null
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)
    
###################################################################################################################################
   
  #(3): TAY_Col_Chr_Ind (Knipovich, Coleoptera, Chrysomelidae, individual level data)
   
   #Read data
   i=3
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outperform m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)
   
###################################################################################################################################
   
  #(4): TAY_Col_sp_ave (Knipovich, Collembola, sp., averaged data)
   
   #Read data
   i=4
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_linear
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(5): TAY_Col_Sta_ind (Knipovich, Coleoptera, Staphylinidae, individual level data)
   
   #Read data
   i=5
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(6): TAY_Dip_Bol_ind (Knipovich, Diptera, Bolitophilidae, individual level data)
   
   #Read data
   i=6
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outperform m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(7): TAY_Dip_Chi_ind (Knipovich, Diptera, Chironomidae, individual level data)
   
   #Read data
   i=7
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 
   
###################################################################################################################################
   
  #(8): TAY_Dip_Emp_ind (Knipovich, Diptera, Empididae, individual level data)
   
   #Read data
   i=8
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, but it does not outperform m_lm_power. Based on residual patterns and biological
   #theoretical foundations for a power-law model we opt for m_lm_power instead.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(9): TAY_Dip_Mus_ind (Knipovich, Diptera, Muscidae, individual level data)
   
   #Read data
   i=9
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(10): TAY_Dip_Myc_ind (Knipovich, Diptera, Mycetophilidae, individual level data)
   
   #Read data
   i=10
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outperform m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(11): TAY_Dip_Sca_ind (Knipovich, Diptera, Scathophagidae, individual level data)
   
   #Read data
   i=11
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Two datapoints is too few to fit a statistical model.
   
   #Create df_AIC with NAs
   df_AIC <- data.frame(df=rep(NA, 4), logLik=rep(NA,4), AIC=rep(NA,4))
   row.names(df_AIC) <- c("m_lm_null", "m_lm_linear", "m_lm_exponential", "m_lm_power")
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- NA
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- NA
   rm(df_taxa)    
   
###################################################################################################################################
   
  #(12): TAY_Dip_Sci_ind (Knipovich, Diptera, Sciaridae, individual level data)
   
   #Read data
   i=12
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_exponential
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(13): TAY_Dip_Tip_ind (Knipovich, Diptera, Tipulidae, individual level data)
   
   #Read data
   i=13
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on residual patterns and the 
   #biological theoretic foundations for expecting a power-law distribution we opted for the m_lm_power model instead.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(14): TAY_Dip_Tri_ind (Knipovich, Diptera, Trichoceridae, individual level data)
   
   #Read data
   i=14
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(15): TAY_Hym_Ich_ind (Knipovich, Hymenoptera, Ichneumonidae, individual level data)
   
   #Read data
   i=15
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)    
   
###################################################################################################################################
   
  #(16): TAY_Hym_Mym_ind (Knipovich, Hymenoptera, Mymaridae, individual level data)
   
   #Read data
   i=16
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_null
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)       
   
 ###################################################################################################################################
   
  #(17): TAY_Hym_Ten_ind (Knipovich, Hymenoptera, Tenthredinidae, individual level data)
   
   #Read data
   i=17
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential. Based on residual patterns and biological 
   #theoretic foundations we prefer model m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)       
   
###################################################################################################################################
   
  #(18): ZAC_Aca_sp._ave (Zackenberg, Acari, sp., averaged data)
   
   #Read data
   i=18
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)      
   
###################################################################################################################################
   
  #(19): ZAC_Aca_sp._ind (Zackenberg, Acari, sp., individual level data)
   
   #Read data
   i=19
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on residual patterns and biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)      
   
###################################################################################################################################
   
  #(20): ZAC_Ara_Dic_ind (Zackenberg, Araneae, Dictynidae, individual level data)
   
   #Read data
   i=20
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on residual patterns and 
   #biological theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)         
   
###################################################################################################################################
   
  #(21): ZAC_Ara_Lin_ind (Zackenberg, Araneae, Linyphiidae, individual level data)
   
   #Read data
   i=21
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)         
   
###################################################################################################################################
   
  #(22): ZAC_Ara_Lyc_ind (Zackenberg, Araneae, Lycosidae, individual level data)
   
   #Read data
   i=22
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)       
   
###################################################################################################################################
   
  #(23): ZAC_Ara_Tho_ind (Zackenberg, Araneae, Thomisidae, individual level data)
   
   #Read data
   i=23
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC, although it does not outcompete model m_lm_power. Based on residual patterns and biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################
   
 #(24): ZAC_Col_sp._ave (Zackenberg, Collembola, sp., averaged data)
   
   #Read data
   i=24
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is the best supported model although it does not outcompete m_lm_linear.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################
   
  #(25): ZAC_Dip_Ant_ind (Zackenberg, Diptera, Anthomyiidae, individual level data)
   
   #Read data
   i=25
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_exponential
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################
   
  #(26): ZAC_Dip_Cer_ave (Zackenberg, Diptera, Ceratopogonidae, averaged data)
   
   #Read data
   i=26
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Remove extreme outlier
   df_taxa <- df_taxa[-14,]
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC, but it does not outcompete m_lm_exponential nor model m_lm_power. Based on residual patterns
   #and biological theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################

 #(27): ZAC_Dip_Chi_ave (Zackenberg, Diptera, Chironomidae, averaged data)
   
   #Read data
   i=27
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_linear
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################
   
  #(28): ZAC_Dip_Chi_ind (Zackenberg, Diptera, Chironomidae, individual level data)
   
   #Read data
   i=28
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)     
   
###################################################################################################################################
   
  #(29): ZAC_Dip_Cul_ind (Zackenberg, Diptera, Culicidae, individual level data)
   
   #Read data
   i=29
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC although it does not outcompete m_lm_exponential
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(30): ZAC_Dip_Emp_ind (Zackenberg, Diptera, Empididae, individual level data)
   
   #Read data
   i=30
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC although it does not outcompete m_lm_power. Based on residual patterns and biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(31): ZAC_Dip_Mus_ind (Zackenberg, Diptera, Muscidae, individual level data)
   
   #Read data
   i=31
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
 #(32): ZAC_Dip_Myc_ind (Zackenberg, Diptera, Mycetophilidae, individual level data)
   
   #Read data
   i=32
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(33): ZAC_Dip_Pho_ind (Zackenberg, Diptera, Phoridae, individual level data)
   
   #Read data
   i=33
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_linear
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(34): ZAC_Dip_Sca_ind (Zackenberg, Diptera, Scathophagidae, individual level data)
   
   #Read data
   i=34
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on residual patterns and biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(35): ZAC_Dip_Sci_ave (Zackenberg, Diptera, Sciaridae, averaged data)
   
   #Read data
   i=35
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
 #(36): ZAC_Dip_Syr_ind (Zackenberg, Diptera, Syrphidae, individual level data)
   
   #Read data
   i=36
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, although it does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(37): ZAC_Dip_Tac_ave (Zackenberg, Diptera, Tachinidae, averaged data)
   
   #Read data
   i=37
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on residual patterns and biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
  #(38): ZAC_Hym_Bom_ind (Zackenberg, Hymenoptera, Apidae, individual level data)
   
   #Read data
   i=38
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, although it does not outcompete m_lm_power. Based on  biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)   
   
###################################################################################################################################
   
 #(39): ZAC_Hym_Bra_ind (Zackenberg, Hymenoptera, Braconidae, individual level data)
   
   #Read data
   i=39
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_linear is best supported based on AIC, although it does not outcompete m_lm_power and m_lm_exponential. Based on  biological
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 

###################################################################################################################################
   
  #(40): ZAC_Hym_Cha_ind (Zackenberg, Hymenoptera, Chalcidoidea, individual level data)
   
   #Read data
   i=40
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_null #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 

###################################################################################################################################
   
  #(41): ZAC_Hym_Ich_ind (Zackenberg, Hymenoptera, Ichneumonidae, individual level data)
   
   #Read data
   i=41
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)  
   
###################################################################################################################################
   
  #(42): ZAC_Lep_Lar_ind (Zackenberg, Lepidoptera, Larvae sp., individual level data)
   
   #Read data
   i=42
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_exponential is best supported based on AIC, but does not outcompete m_lm_power. Based on biological 
   #theoretic foundations we prefer m_lm_power.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_power # get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) 
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 
   
###################################################################################################################################
   
  #(43): ZAC_Lep_Noc_ind (Zackenberg, Lepidoptera, Noctuidae, individual level data)
   
   #Read data
   i=43
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_null
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 
   
###################################################################################################################################
   
 #(44): ZAC_Lep_Nym_ind (Zackenberg, Lepidoptera, Nymphalidae, individual level data)
   
   #Read data
   i=44
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_power is best supported based on AIC, but does not outcompete m_lm_exponential.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])  #m_lm_power
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict) 
   
###################################################################################################################################
   
  #(45): ZAC_Lep_Pie_ind (Zackenberg, Lepidoptera, Pieridae, individual level data)
   
   #Read data
   i=45
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1]) #m_lm_null
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)
   
###################################################################################################################################
   
  #(46): ZAC_Thy_sp._ind (Zackenberg, Thysanoptera, sp., individual level data)
   
   #Read data
   i=46
   df_taxa <- df[df$name==taxa[i],]
   if(any(df_taxa$weight_level=="average") &  any(df_taxa$weight_level=="individual")){print("WARNING: weight measures based on individual measures and averages. Separate these datasets!")}
   
   #Visually inspect data
   ggplot(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg), col=name))+
     geom_point()+
     geom_smooth(method='lm', formula= y~x, se=FALSE)+
     xlab("log length (mm)")+
     ylab("log bodymass (mg)")+
     theme_classic()
   
   #Fit the following models:
   #(0) Intercept only model (Null model)
   #(1) Linear model on untransformed data
   #(2) Bias-corrected linear model on log-transformed data (i.e. bias-corrected Exponential function)
   #(3) Bias-corrected linear model on log-log transformed data (i.e. bias-corrected Power function)
   m_lm_null <- lm(weight_mg ~ 1, data=df_taxa)
   m_lm_linear <- lm(weight_mg~length_mm, data=df_taxa)
   m_lm_exponential <- lm(log(weight_mg)~length_mm, data=df_taxa)
   m_lm_power <- lm(log(weight_mg)~log(length_mm), data=df_taxa)
   
   #Compare adjusted AIC values
   df_AIC <- data.frame(model.sel(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power, rank=AIC))[,c("df", "logLik", "AIC")]
   #Note that we cannot use AIC to compare the models with an un-transformed response variable to the ones with a log-transformed variable.
   #Instead we need to adjust our AIC measure for this by adding minus twice the logarithm of the Jacobian to our AIC value (Akaike 1978)
   index <- rownames(df_AIC) %in% c("m_lm_exponential", "m_lm_power" )
   df_AIC[index, "AIC"] <- df_AIC[index, "AIC"] + 2*sum(log(df_taxa$weight_mg))
   df_AIC <- df_AIC[order(df_AIC$AIC),] ; df_AIC  ; rm(index)
   #Model m_lm_null is best supported based on AIC.
   
   #Inspect residual patterns (normalized Pearson residuals) of all models
   
   #Versus fitted values
   plot(resid(m_lm_linear, type = "pearson")~fitted(m_lm_linear))
   plot(resid(m_lm_exponential, type = "pearson")~fitted(m_lm_exponential))
   plot(resid(m_lm_power, type = "pearson")~fitted(m_lm_power))
   
   #Versus length_mm
   plot(resid(m_lm_linear, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_exponential, type = "pearson")~df_taxa$length_mm)
   plot(resid(m_lm_power, type = "pearson")~df_taxa$length_mm)
   
   #Check normality
   qqnorm(resid(m_lm_linear, type="pearson")) ; qqline(resid(m_lm_linear, type="pearson"))
   qqnorm(resid(m_lm_exponential, type="pearson")) ; qqline(resid(m_lm_exponential, type="pearson"))
   qqnorm(resid(m_lm_power, type="pearson")) ; qqline(resid(m_lm_power, type="pearson"))
   
   #Store model predictions on untransformed scale
   df_modpredict <- data.frame(length_mm=seq(min(df_taxa$length_mm), max(df_taxa$length_mm),  length.out = 250))
   df_modpredict$m_lm_null <- predict(m_lm_null, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_linear <- predict(m_lm_linear, newdata=df_modpredict, level = 0)
   df_modpredict$m_lm_exponential <- exp(predict(m_lm_exponential, newdata=df_modpredict, level = 0))
   df_modpredict$m_lm_power <- exp(predict(m_lm_power, newdata=df_modpredict, level = 0))
   
   #Correct model predictions for bias in log backtransformation using Smearing correction factor
   SF_m_lm_exponential <- mean(exp(residuals(m_lm_exponential)))
   df_modpredict$m_lm_exponential_corrected <- df_modpredict$m_lm_exponential * SF_m_lm_exponential
   SF_m_lm_power <- mean(exp(residuals(m_lm_power)))
   df_modpredict$m_lm_power_corrected <- df_modpredict$m_lm_power * SF_m_lm_power
   rm(SF_m_lm_exponential, SF_m_lm_power)
   
   #Restrict model predictions to positive values to prevent errors when log transforming
   df_modpredict <- df_modpredict[!apply(df_modpredict[,c("m_lm_null", "m_lm_linear", "m_lm_exponential_corrected", "m_lm_power_corrected")], 1, function(x){{any(x<0)}}), ]
   
   #Plot model predictions on log-log scale
   p <- ggplot()+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_null)), col="grey", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_linear)), col="black", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential)), col="blue", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_exponential_corrected)), col="blue", lwd=1, lty=2)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power)), col="red", lwd=1.25)+
     geom_line(data=df_modpredict, aes(x=log(length_mm), y=log(m_lm_power_corrected)), col="red", lwd=1, lty=2)+
     geom_point(data=df_taxa, aes(x=log(length_mm), y=log(weight_mg)), pch=21, size=2, col="black")+
     xlab("log Bodylength (mm)")+
     ylab("log Bodymass (mg)")+
     #ylim(c(log(min(df_taxa$weight_mg)), log(max(df_taxa$weight_mg))))+
     theme_tom()
   
   #Save best model
   m_best <- m_lm_null #get(rownames(df_AIC[which(df_AIC$AIC==min(df_AIC$AIC)),])[1])
   rm(m_lm_null, m_lm_linear, m_lm_exponential, m_lm_power)
   
   #Store all relevant objects in a list
   list_taxa[[i]][[1]] <- df_taxa$name[1]
   list_taxa[[i]][[2]] <- df_taxa
   list_taxa[[i]][[3]] <- nrow(df_taxa)
   list_taxa[[i]][[4]] <- m_best
   list_taxa[[i]][[5]] <- df_AIC
   list_taxa[[i]][[6]] <- p
   rm(df_taxa, m_best, df_AIC, p, df_modpredict)

  #Save list_taxa
   saveRDS(list_taxa, file="Versluijs2022_Output_1-LengthBiomassRegressions_ModelSelectionList.RData")
   #list_taxa <- readRDS("Versluijs2022_Output_1-LengthBiomassRegressions_ModelSelectionList.RData")
  
###################################################################################################################################
   
#(III): Create a model selection table

###################################################################################################################################

  #Create a model selection table
   model_sel <- do.call(rbind, sapply(list_taxa,`[`,5))
   taxon <- rep(unlist(sapply(list_taxa,`[`,1)), each=4)
   n <- rep(unlist(sapply(list_taxa,`[`,3)), each=4)
   model_sel <- cbind(data.frame(taxon=taxon, n=n), model_sel)
   
  #Restrict to those families which we present in our manuscript (i.e. n>=8) 
   model_sel <- model_sel[model_sel$n>=8 | model_sel$taxon=="ZAC_Aca_sp._ave", ]
   
  #Add model formula
   model_sel$model <- NA
   model_sel$model[grepl("m_lm_null", row.names(model_sel))] <- "Weight = B0"
   model_sel$model[grepl("m_lm_linear", row.names(model_sel))] <- "Weight = B0 + B1 * length"
   model_sel$model[grepl("m_lm_exponential", row.names(model_sel))] <- "ln(Weight/SF) = B0 + B1 * length"
   model_sel$model[grepl("m_lm_power", row.names(model_sel))] <- "ln(Weight/SF) = B0 + B1 * ln(length)"
   
  #Add location
   model_sel$location <- NA
   model_sel$location[grepl("TAY", model_sel$taxon)] <- "KNP"
   model_sel$location[grepl("ZAC", model_sel$taxon)] <- "ZAC"
  
  #Add weight level
   model_sel$level <- NA
   model_sel$level[grepl("ind", model_sel$taxon)] <- "ind"
   model_sel$level[grepl("ave", model_sel$taxon)] <- "avg" 
   
  #Add Order
   model_sel$order <- NA
   model_sel$order[grepl("Aca", model_sel$taxon)] <- "Acari" 
   model_sel$order[grepl("Ara", model_sel$taxon)] <- "Araneae" 
   model_sel$order[grepl("Col", model_sel$taxon)] <- "Coleoptera" 
   model_sel$order[grepl("Col_sp", model_sel$taxon)] <- "Collembola" 
   model_sel$order[grepl("Dip", model_sel$taxon)] <- "Diptera"
   model_sel$order[grepl("Hym", model_sel$taxon)] <- "Hymenoptera" 
   model_sel$order[grepl("Lep", model_sel$taxon)] <- "Lepidoptera" 
   model_sel$order[grepl("Thy", model_sel$taxon)] <- "Thysanoptera" 
   
  #Add Family
   model_sel$family <- NA
   model_sel$family[grepl("Ara_Lin", model_sel$taxon)] <- "Linyphiidae" 
   model_sel$family[grepl("Col_Car", model_sel$taxon)] <- "Carabidae" 
   model_sel$family[grepl("Col_Chr", model_sel$taxon)] <- "Chrysomelidae" 
   model_sel$family[grepl("Col_sp.", model_sel$taxon)] <- "sp." 
   model_sel$family[grepl("Col_Sta", model_sel$taxon)] <- "Staphylinidae" 
   model_sel$family[grepl("Dip_Bol", model_sel$taxon)] <- "Bolitophilidae" 
   model_sel$family[grepl("Dip_Chi", model_sel$taxon)] <- "Chironomidae" 
   model_sel$family[grepl("Dip_Emp", model_sel$taxon)] <- "Empididae" 
   model_sel$family[grepl("Dip_Mus", model_sel$taxon)] <- "Muscidae" 
   model_sel$family[grepl("Dip_Myc", model_sel$taxon)] <- "Mycetophilidae" 
   model_sel$family[grepl("Dip_Sci", model_sel$taxon)] <- "Sciaridae" 
   model_sel$family[grepl("Dip_Tip", model_sel$taxon)] <- "Tipulidae" 
   model_sel$family[grepl("Dip_Tri", model_sel$taxon)] <- "Trichoceridae" 
   model_sel$family[grepl("Hym_Ich", model_sel$taxon)] <- "Ichneumonidae" 
   model_sel$family[grepl("Hym_Ten", model_sel$taxon)] <- "Tenthredinidae" 
   model_sel$family[grepl("Aca_sp.", model_sel$taxon)] <- "sp." 
   model_sel$family[grepl("Ara_Dic", model_sel$taxon)] <- "Dictynidae" 
   model_sel$family[grepl("Ara_Lyc", model_sel$taxon)] <- "Lycosidae" 
   model_sel$family[grepl("Ara_Tho", model_sel$taxon)] <- "Thomisidae" 
   model_sel$family[grepl("Dip_Ant", model_sel$taxon)] <- "Anthomyiidae"    
   model_sel$family[grepl("Dip_Cer", model_sel$taxon)] <- "Ceratopogonidae" 
   model_sel$family[grepl("Dip_Cul", model_sel$taxon)] <- "Culicidae" 
   model_sel$family[grepl("Dip_Pho", model_sel$taxon)] <- "Phoridae" 
   model_sel$family[grepl("Dip_Sca", model_sel$taxon)] <- "Scathophagidae" 
   model_sel$family[grepl("Dip_Syr", model_sel$taxon)] <- "Syrphidae" 
   model_sel$family[grepl("Dip_Tac", model_sel$taxon)] <- "Tachinidae" 
   model_sel$family[grepl("Lep_Nym", model_sel$taxon)] <- "Nymphalidae" 
   
  #Change order of columns
   model_sel <- model_sel[,c("order", "family", "n", "df", "logLik", "AIC", "level", "model", "location")]
   
  #Sort by Order, Family, location, AIC
   model_sel <- model_sel[with(model_sel, order(order, family, location, AIC)), ]
   
  #adjust rownames
   row.names(model_sel) <- 1:nrow(model_sel)
  
  #Round loglikelihood and AIC
   model_sel$AIC <- round(model_sel$AIC, 3)
   model_sel$logLik <- round(model_sel$logLik, 3)
   
  #Save model selection dataframe
   write.csv(model_sel, file="Versluijs2022_Output_1-LengthBiomassRegressions_TableS2.csv", row.names=F)
   
###################################################################################################################################
   
#(IV): Calculate 95% quantile bootstrapped confidence intervals for model predictions and for regression coefficients   
   
###################################################################################################################################

   #To provide a measure of uncertainty for the fitted allometric equations we calculated 95% confidence intervals for model 
   #predictions and regression coefficients using non-parametric (case) bootstrapping. We opted for a non-parametric approach 
   #because normality and homoscedasticity assumptions were not always met. We calculated quantile 95% confidence intervals. 
   #Generally, BCa confidence intervals would be preferred (Efron and Narasimhan 2021), but we encountered some issues in 
   #combination with influential datapoints and therefore preferred quantile confidence intervals.

  #Specify several bootstrap parameters
   nsim=10000 #number of bootstrap iterations
   boot_threshold=20 #minimum sample size for bootstrapping
   mod_threshold=8 #minimum sample size for plotting of statistical model
   confint_type="quantile" #"quantile" or "bca"
   boot_type="case" #"fixef", "residual" or "case" (note that 'fixef' does not calculate smearing factors)
   data_x="length_mm" #predictor variable for all allometric models
   data_y="weight_mg" #response variable for all allometric models
   
  #We only plot model predictions on top of the raw data if the sample size is at least equal to mod_threshold. 
  #Create an index variable that indicates which data meet this requirement.
   index <- lapply(list_taxa, function(l) l[[3]]) >= mod_threshold
   
  #Some data are based on averages from a much larger dataset. We still plot our model if the initial sample size before 
  #averaging was at least equal to mod_threshold. We therefore update our indicator variable to add these cases.
   names_avg <- df_n_avg$name[ df_n_avg$n_before >= mod_threshold]
   index_names <- lapply(list_taxa, function(l) l[[1]]) %in% names_avg
   index <- which(index | index_names)

  #Loop through all taxon groups and calculate bootstrapped confidence intervals if applicable. The code below first checks
  #whether each taxon occurs in the indicator variable 'index' which indicates whether we want to plot the fitted
  #model on top of the raw data (i.e. we do not do this if the sample size is < mod_threshold). If it does not occur in 
  #'index' then we only plot the raw datapoints. If it does occur in 'index' then we subsequently check whether the 
  #number of datapoints for this taxon is larger than the minimum sample size for bootstrapping (i.e. n > boot_threshold).
  #If so, then we use case bootstrapping to calculate 95% CI of model parameters and predictions and plot the fitted model
  #including these boostrapped predictions on top of the raw data. If the minimum sample size for boostrapping was not met
  #then we only plot the fitted model on top of the raw datapoints. In any scenario, all output is added to 'list_taxa'
  #(e.g. the plots, a summarizing dataframe with 95% CI of model coefficients, dataframe with individual boostrapping 
  #predictions)   
   for(i in 1:length(list_taxa)){

     #Minimum sample size for plotting of model is met:
     if(i %in% index){
       
      #Extract objects of current taxon (i)
       name <- list_taxa[[i]][[1]]
       df_taxon <- list_taxa[[i]][[2]]
       N_taxon <- list_taxa[[i]][[3]]
       mod_taxon <- list_taxa[[i]][[4]]
       df_AIC_taxon <- list_taxa[[i]][[5]]
       p <- list_taxa[[i]][[6]]
      
      #Plot original plot
       print(p)

      #Specify dataframe
       data <- df_taxon
      
      #Specify newdata dataframe (used to make model predictions)
       newdata <- data.frame(length_mm=seq(min(data$length_mm), max(data$length_mm),  length.out = 250))
      
      #Specify model (mod)
       mod <- mod_taxon
      
      #Determine model formula (mod_func)
       if(class(mod)[1]=="lm"){mod_form <- mod$call$formula}
       if(mod_form=="log(weight_mg) ~ length_mm"){mod_func <- function(length_mm, a, b, ...){exp(a + b*length_mm)}}
       if(mod_form=="log(weight_mg) ~ log(length_mm)"){mod_func <- function(length_mm, a, b, ...){exp(a + b*log(length_mm))}}
       if(mod_form=="weight_mg ~ length_mm"){mod_func <- function(length_mm, a, b, ...){a + b*length_mm}}
       if(mod_form=="weight_mg ~ 1"){mod_func <- function(a, ...){a}}      
      
      #Indicate if the response variable of mod is log-transformed (if so, then a smearing correction will be applied)
       if(grepl("log(weight_mg)", toString(mod_form), fixed = TRUE)){ is_y_log_transformed = TRUE }
       if(!grepl("log(weight_mg)", toString(mod_form), fixed = TRUE)){ is_y_log_transformed = FALSE }
      
      #Extract coefficients of original model ('mod')
       if(!(mod_form=="weight_mg ~ 1")){
        
         #If y-variable log-transformed
         if(is_y_log_transformed == TRUE){
          mod_coef <- data.frame(X1=c(coef(mod)[1], coef(mod)[2], mean(exp(residuals(mod))) ))
          row.names(mod_coef) <- c("a", "b", "SF")
          }
        
         #If y-variable is not log-transformed
         if(is_y_log_transformed == FALSE){
          mod_coef <- data.frame(X1=c(coef(mod)[1], coef(mod)[2], NA ))
          row.names(mod_coef) <- c("a", "b", "SF")
          }
        
        }
       if(mod_form=="weight_mg ~ 1"){
         mod_coef <- data.frame(X1=c(coef(mod)[1], NA, NA))
         row.names(mod_coef) <- c("a", "b", "SF")
         }
      
      #Store fitted values from original model ('mod')
       mod_predict <- data.frame(x=newdata[,"length_mm"])
       mod_predict$y <- mod_func(newdata[,"length_mm"], a=coef(mod)[1], b=coef(mod)[2])
    
      #Correct predictions for log-transformation of response variable using smearing factor (if necessary)
       if(is_y_log_transformed){
        SF_mod <- mean(exp(residuals(mod)))
        mod_predict$y <- mod_predict$y * SF_mod
       }
       
      #Add sample size before the averaging process in brackets (if available)
       if(name %in% names_avg){
        N_output <- paste0(N_taxon, " (", df_n_avg$n_before[df_n_avg$name==name] ,")")
        }
       if(!(name %in% names_avg)){
        N_output <- N_taxon
        }
    
      #(I): No bootstrapping when sample size < boot_threshold
       if(N_taxon < boot_threshold){

         #Print progress
          print(paste0("Start: ", i, " - ", name, " - n=", N_output)) 
         
         #Set boot_converged and boot_n to NA
          boot_coef <- NA
          boot_predict <- NA
          boot_converged <- NA
          boot_n <- NA
          boot_coef_CI <- NA
          boot_predict_CI <- NA
        
         #Plot original model predictions without bootstrapping confidence intervals
          list_taxa[[i]][[7]] <- local({
            
            #recreate required local variables
            mod_predict=mod_predict
            data=data
            N_output=N_output
            i=i
            
            #Create sample size label for plotting
            label <- textGrob(label=paste0("N=", N_output), x=0.05, y=0.95, just=c("left", "top"), gp = gpar(fontsize = 15))  
           
            #Create title
            title <- paste0(data$order[1], " ", data$taxon[1], " - ", data$Area[1])
            
            #Create plot for TAY
            if(data$Area[1] == "TAY"){
              p_taxon <- ggplot()+
                geom_line(data=mod_predict, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
                geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=1, size=2, stroke=1.25)+
                #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                xlab("Bodylength (mm)")+
                ylab("Bodymass (mg)")+
                annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
                # annotate("text", 
                #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
                #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
                #          label=paste0("N=", N_Output), size=6)+
                ggtitle(title)+
                theme_tom()
                }
            
            #Create plot for ZAC
            if(data$Area[1] == "ZAC"){
              p_taxon <- ggplot()+
                geom_line(data=mod_predict, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
                geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=2, size=2, stroke=1.25)+
                #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                xlab("Bodylength (mm)")+
                ylab("Bodymass (mg)")+
                annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
                # annotate("text", 
                #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
                #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
                #          label=paste0("N=", N_Output), size=6)+
                ggtitle(title)+
                theme_tom()
                }
       
            #Output plot to global environment
            print(p_taxon)
            
            })

         #Create a summarizing dataframe
          df_summary <- data.frame(Order=as.character(unique(df_taxon$order)),
                                   Taxon=as.character(unique(df_taxon$taxon)),
                                   Stage=NA,
                                   N=N_output,
                                   min_mm=min(df_taxon$length_mm),
                                   max_mm=max(df_taxon$length_mm),
                                   B0=mod_coef[row.names(mod_coef)=="a",],
                                   B0_lci=NA,
                                   B0_uci=NA,
                                   B1=mod_coef[row.names(mod_coef)=="b",],
                                   B1_lci=NA,
                                   B1_uci=NA,
                                   SF=mod_coef[row.names(mod_coef)=="SF",],
                                   SF_lci=NA,
                                   SF_uci=NA,
                                   Length="mm",
                                   Weight="mg",
                                   Weight_level=as.character(unique(df_taxon$weight_level)),
                                   Model_mm_mg=as.character(c(mod_form)),
                                   Location=as.character(unique(df_taxon$Area)))  
          
          
         }
      
      #(II): Case bootstrapping when sample size >= boot_threshold
       if(N_taxon >= boot_threshold){
        
        #Print progress
         print(paste0("Start bootstrap: ", i, " - ", name, " - n=", N_output))  
         
        #Run bootstrapping function
         l_boot_out <- f_bootstrap_Tom(mod=mod, mod_func=mod_func, data=data, data_x=data_x, data_y=data_y, newdata=newdata, nsim=nsim, boot_type=boot_type)
        
        #Extract bootstrapping output from list
         boot_coef <- l_boot_out[['coefficients']]
         boot_predict <- l_boot_out[['fitted']]
         boot_converged <- l_boot_out[['converged']]
         boot_n <- l_boot_out[['nsim']]
        
        #If the y-variable was log-transformed, correct bootstrapped predictions using a smearing factor
         if(is_y_log_transformed){

           #Adjust bootstrapped predictions
            SF <- unlist(boot_coef[rownames(boot_coef)=="SF",])
            boot_predict <- data.frame(as.matrix(boot_predict) %*% diag(SF))

            }
        
        #Calculate 95% confidence intervals of bootstrapped model coefficients
         boot_coef_CI <- get_CI(y=boot_coef, pref="", type=confint_type, boot_confint=0.95)
         boot_coef_CI$boot_mean <- apply(boot_coef, 1, FUN=function(x){mean(na.omit(x))})
        
        #Calculate 95% confidence intervals of bootstrapped model predictions (of all nsim runs)
         boot_predict_CI <- get_CI(y=boot_predict, pref="", type=confint_type, boot_confint=0.95)
         boot_predict_CI$boot_mean <- c(t(apply(boot_predict, 1, FUN=function(x){mean(na.omit(x))})))
        
        #Take a random subset of 1000 bootstrapping runs to make plotting of all lines feasible 
         boot_predict <- boot_predict[,sample(x = 1:10000, size = 1000, replace = FALSE)]
         
        #Reshape dataframe boot_predict to allow plotting of individual lines
         if(!(mod_form=="weight_mg ~ 1")){
          boot_predict$x <- newdata[,"length_mm"]
          boot_predict <- melt(boot_predict, id="x")
          names(boot_predict) <- c("x", "sim", "y")  
          }
         if(mod_form=="weight_mg ~ 1"){
          boot_predict <- boot_predict[rep(1, length(newdata[,"length_mm"])), ]
          boot_predict$x <- newdata[,"length_mm"]
          boot_predict <- melt(boot_predict, id="x")
          names(boot_predict) <- c("x", "sim", "y")  
          }

        #Plot bootstrapped model predictions with raw datapoints on a log-log scale
         list_taxa[[i]][[7]] <- local({
           
           #recreate required local variables
            boot_predict=boot_predict
            mod_predict=mod_predict
            boot_predict_CI=boot_predict_CI
            data=data
            N_output=N_output
            i=i
            
           #Create sample size label for plotting
            label <- textGrob(label=paste0("N=", N_output), x=0.05, y=0.95, just=c("left", "top"), gp = gpar(fontsize = 15))
            
           #Create title
            title <- paste0(data$order[1], " ", data$taxon[1], " - ", data$Area[1])  
            
           #Create plot for TAY  
            if(data$Area[1] == "TAY"){
             
              p_taxon <- ggplot()+
                geom_line(data=boot_predict, aes(x=x, y=y, group=sim), col="red", lty=1, lwd=0.75, alpha=0.01)+
                #geom_ribbon(aes(x=mod_predict$x, ymin=boot_predict_CI[,1], ymax=boot_predict_CI[,2]), colour=NA, alpha=0.75, fill="grey75")+
                #geom_line(aes(x=mod_predict$x, y=boot_predict_CI$boot_mean), col="darkblue", lwd=1.5, alpha=0.5)+
                geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=1, size=2, stroke=1.25)+
                geom_line(data=mod_predict, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
                geom_line(aes(x=mod_predict$x, y=boot_predict_CI[,1]), col="black", lty=3, lwd=1, alpha=1)+
                geom_line(aes(x=mod_predict$x, y=boot_predict_CI[,2]), col="black", lty=3, lwd=1, alpha=1)+
                #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                xlab("Bodylength (mm)")+
                ylab("Bodymass (mg)")+
                annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
                # annotate("text", 
                #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
                #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
                #          label=paste0("N=", N_output), size=6)+
                ggtitle(title)+
                theme_tom()
               
              }
            
           #Create plot for ZAC
            if(data$Area[1] == "ZAC"){
              
              p_taxon <- ggplot()+
                geom_line(data=boot_predict, aes(x=x, y=y, group=sim), col="red", lty=1, lwd=0.75, alpha=0.01)+
                #geom_ribbon(aes(x=mod_predict$x, ymin=boot_predict_CI[,1], ymax=boot_predict_CI[,2]), colour=NA, alpha=0.75, fill="grey75")+
                #geom_line(aes(x=mod_predict$x, y=boot_predict_CI$boot_mean), col="darkblue", lwd=1.5, alpha=0.5)+
                geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=2, size=2, stroke=1.25)+
                geom_line(data=mod_predict, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
                geom_line(aes(x=mod_predict$x, y=boot_predict_CI[,1]), col="black", lty=3, lwd=1, alpha=1)+
                geom_line(aes(x=mod_predict$x, y=boot_predict_CI[,2]), col="black", lty=3, lwd=1, alpha=1)+
                #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
                scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
                xlab("Bodylength (mm)")+
                ylab("Bodymass (mg)")+
                annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
                # annotate("text", 
                #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
                #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
                #          label=paste0("N=", N_output), size=6)+
                ggtitle(title)+
                theme_tom()
    
              }
            
           #Output plot to global environment
           print(p_taxon)
        
           })
         
         #Create a summarizing dataframe
         if(confint_type=="bca"){CI_affix <- "_bca_"}
         if(confint_type=="quantile"){CI_affix <- "_quantile_"}
         df_summary <- data.frame(Order=as.character(unique(df_taxon$order)),
                                  Taxon=as.character(unique(df_taxon$taxon)),
                                  Stage=NA,
                                  N=N_output,
                                  min_mm=min(df_taxon$length_mm),
                                  max_mm=max(df_taxon$length_mm),
                                  B0=mod_coef[row.names(mod_coef)=="a",],
                                  B0_lci=boot_coef_CI[row.names(boot_coef_CI)=="a", paste0(CI_affix, 'lci')],
                                  B0_uci=boot_coef_CI[row.names(boot_coef_CI)=="a", paste0(CI_affix, 'uci')],
                                  B1=mod_coef[row.names(mod_coef)=="b",],
                                  B1_lci=boot_coef_CI[row.names(boot_coef_CI)=="b", paste0(CI_affix, 'lci')],
                                  B1_uci=boot_coef_CI[row.names(boot_coef_CI)=="b", paste0(CI_affix, 'uci')],
                                  SF=mod_coef[row.names(mod_coef)=="SF",],
                                  SF_lci=boot_coef_CI[row.names(boot_coef_CI)=="SF", paste0(CI_affix, 'lci')],
                                  SF_uci=boot_coef_CI[row.names(boot_coef_CI)=="SF", paste0(CI_affix, 'uci')],
                                  Length="mm",
                                  Weight="mg",
                                  Weight_level=as.character(unique(df_taxon$weight_level)),
                                  Model_mm_mg=as.character(c(mod_form)),
                                  Location=as.character(unique(df_taxon$Area)))
         
         }

      #Add all statistics to list_taxa
       list_taxa[[i]][[8]] <- mod_coef #original coefficients of best supported model
       list_taxa[[i]][[9]] <- boot_coef_CI #bootstrapped coefficients of best supported model
       list_taxa[[i]][[10]] <- boot_converged #number of converged models
       list_taxa[[i]][[11]] <- boot_n #number of bootstrapping runs
       list_taxa[[i]][[12]] <- df_summary #summary of best model including bootstrapped 95% CI for model coefficients 
       list_taxa[[i]][[13]] <- mod_predict #predictions of original model
       list_taxa[[i]][[14]] <- boot_predict_CI #95% CI of boostrapped predictions
       list_taxa[[i]][[15]] <- boot_predict #predictions of each boostrapping run

     }
     
     #Minimum sample size for plotting of model is not met:
     if(!(i %in% index)){
       
       #Extract objects of current taxon (i)
       name <- list_taxa[[i]][[1]]
       df_taxon <- list_taxa[[i]][[2]]
       N_taxon <- list_taxa[[i]][[3]]
     
       #Specify dataframe for current taxon
       data <- df_taxon

       #Add sample size before averaging in brackets (if applicable)
       if(name %in% names_avg){
         N_output <- paste0(N_taxon, " (", df_n_avg$n_before[df_n_avg$name==name] ,")")
         }
       if(!(name %in% names_avg)){
         N_output <- N_taxon
         }
       
       #Print progress
       print(paste0("Start: ", i, " - ", name, " - n=", N_output)) 
       
       #Plot raw data
       list_taxa[[i]][[7]] <- local({
         
         #recreate required local variables
         data=data
         N_output=N_output
         i=i
         
         #Create sample size label for plotting
         label <- textGrob(label=paste0("N=", N_output), x=0.05, y=0.95, just=c("left", "top"), gp = gpar(fontsize = 15))
         
         #Create title
         title <- paste0(data$order[1], " ", data$taxon[1], " - ", data$Area[1])
         
         #Create plot for TAY
         if(data$Area[1] == "TAY"){
         
           p_taxon <- ggplot()+
             geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=1, size=2, stroke=1.25)+
             #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
             #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
             scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
             scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
             xlab("Bodylength (mm)")+
             ylab("Bodymass (mg)")+
             annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
             # annotate("text", 
             #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
             #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
             #          label=paste0("N=", N_output), size=6)+
             ggtitle(title)+
             theme_tom()
         
           }
         
         #Create plot for ZAC
         if(data$Area[1] == "ZAC"){
           
           p_taxon <- ggplot()+
             geom_point(data=data, aes(x=length_mm, y=weight_mg), col="black", alpha=0.25, pch=2, size=2, stroke=1.25)+
             #scale_x_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
             #scale_y_continuous(trans = scales::log_trans(), breaks = scales::breaks_log(n = 6, base=exp(1)))+
             scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
             scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
             xlab("Bodylength (mm)")+
             ylab("Bodymass (mg)")+
             annotation_custom(label, xmin = -Inf, xmax = Inf, ymin = -Inf, ymax = Inf)+
             # annotate("text", 
             #          x=min(data$length_mm) + 0.02*(max(data$length_mm) - min(data$length_mm)),
             #          y=max(data$weight_mg) - 0.02*(max(data$weight_mg) - min(data$weight_mg)), 
             #          label=paste0("N=", N_output), size=6)+
             ggtitle(title)+
             theme_tom()
           
           }
         
         #Output plot to global environment
         print(p_taxon)
         
       })
       
       #Create a summarizing dataframe
       df_summary <- data.frame(Order=as.character(unique(df_taxon$order)),
                                Taxon=as.character(unique(df_taxon$taxon)),
                                Stage=NA,
                                N=N_output,
                                min_mm=min(df_taxon$length_mm),
                                max_mm=max(df_taxon$length_mm),
                                B0=NA,
                                B0_lci=NA,
                                B0_uci=NA,
                                B1=NA,
                                B1_lci=NA,
                                B1_uci=NA,
                                SF=NA,
                                SF_lci=NA,
                                SF_uci=NA,
                                Length=NA,
                                Weight=NA,
                                Weight_level=as.character(unique(df_taxon$weight_level)),
                                Model_mm_mg=NA,
                                Location=as.character(unique(df_taxon$Area)))
       
      #Store all elements in a list
       list_taxa[[i]][[8]] <- NA #mod_coef
       list_taxa[[i]][[9]] <- NA #boot_coef_CI
       list_taxa[[i]][[10]] <- NA #boot_converged
       list_taxa[[i]][[11]] <- NA #boot_n
       list_taxa[[i]][[12]] <- df_summary #summary of best model including bootstrapped 95% CI for model coefficients 
       list_taxa[[i]][[13]] <- NA #mod_predict
       list_taxa[[i]][[14]] <- NA #boot_predict_CI
       list_taxa[[i]][[15]] <- NA #boot_predict

     }
  
    }

   #Save the list with objects per taxa
   save(list_taxa, file="Versluijs2022_Output_1-LengthBiomassRegressions_BootstrappingList.RData")
   #load("Versluijs2022_Output_1-LengthBiomassRegressions_BootstrappingList.RData")
   
   #list_taxa now contains 46 elements, where each element represents a different taxonomic group. This can be selected with
   #the first index: e.g. to select the data for Araneae Linyphiidae in Knipovich the first element needs to be selected 
   #list_taxa[[1]][[X]]. Within each element there are 15 different statistics that can be selected using the second index
   #indicated by the X in the previous example. The following data can be extracted: 
       #list_taxa[[i]][[1]] #abbreviated name indicating location_order_family_weightlevel
       #list_taxa[[i]][[2]] #original data for taxonomic group
       #list_taxa[[i]][[3]] #sample size
       #list_taxa[[i]][[4]] #best supported allometric model
       #list_taxa[[i]][[5]] #model selection table
       #list_taxa[[i]][[6]] #plot with fits of all allometric models
       #list_taxa[[i]][[7]] #Plot with fit of best supported model including confidence intervals if applicable
       #list_taxa[[i]][[8]] #coefficients of best supported model
       #list_taxa[[i]][[9]] #bootstrapped coefficients of best supported model
       #list_taxa[[i]][[10]] #number of converged models
       #list_taxa[[i]][[11]] #number of bootstrapping runs
       #list_taxa[[i]][[12]] #summary of best model including bootstrapped 95% CI for model coefficients 
       #list_taxa[[i]][[13]] #predictions of original model
       #list_taxa[[i]][[14]] #95% CI of boostrapped predictions
       #list_taxa[[i]][[15]] #predictions of each boostrapping run

###################################################################################################################################
   
#(V): Summarize the bootstrapping results and plot the data
   
###################################################################################################################################
   
  #(1): Combine dataframes with the best model per taxa
   
   #Extract summarizing dataframes from list_taxa and combine them into a single dataframe
   df_summary <- lapply(list_taxa, function(l) l[[12]])
   df_summary <- do.call("rbind", df_summary)
   
   #Calculate mean smearing factor across all taxa including 95% quantile CI calculated using non-parametric (case) bootstrapping
 
      #Specify the case-sampling function 
       sampfun_case <- function(data) {
     
         #Re-sample dataframe rows (cases) with replacement
          data_bootcase <- data[sample(length(data), replace=TRUE)]
          return(data_bootcase)
     
         } 
   
      #Conduct hierarchical case bootstrapping by iterating the following process 10000 times: (I) use sampfun_case to sample from df_summary
      #with replacement, (II) Calculate the mean smearing factor for each sampled dataset. (III) Store these means in a vector 'y'. (IV) Calculate 
      #95% quantile confidence intervals for this the vector of boostrapped smearing factors 'y'.
       set.seed(23)
       y <- replicate(10000, mean(sampfun_case(data=na.omit(df_summary$SF))))
   
      #Calculated 95% quantile confidence intervals of the stored means
       boot_confint=0.95
       lwr_ci <- (1-boot_confint)/2
       SF_CI_quantile <- t(quantile(na.omit(y), probs=c(lwr_ci, 1-lwr_ci))) #CI_bca <- coxed::bca(na.omit(y), conf.level=boot_confint)
       SF_CI_quantile
       mean(na.omit(df_summary$SF))
       #Thus, the average smearing factor across all taxa is 1.041 [95% CI: 1.027, 1.058]

   #Adjust model formulas to the required format (including Duan's smearing factor)
   df_summary[df_summary$Model_mm_mg=="log(weight_mg) ~ length_mm" & !is.na(df_summary$Model_mm_mg), "Model_mm_mg"] <-  "ln(Weight/SF) = B0 + B1 * length"
   df_summary[df_summary$Model_mm_mg=="log(weight_mg) ~ log(length_mm)" & !is.na(df_summary$Model_mm_mg), "Model_mm_mg"] <- "ln(Weight/SF) = B0 + B1 * ln(length)"
   df_summary[df_summary$Model_mm_mg=="weight_mg ~ length_mm" & !is.na(df_summary$Model_mm_mg), "Model_mm_mg"] <-  "Weight = B0 + B1 * length"
   df_summary[df_summary$Model_mm_mg=="weight_mg ~ 1" & !is.na(df_summary$Model_mm_mg), "Model_mm_mg"] <-  "Weight = B0"
   
   #Add lifestage
   df_summary$Stage <- "Adult"
   df_summary$Stage[df_summary$Taxon=="Larvae sp."] <- "Larvae"
   
   #Round columns
   df_summary$min_mm <- round(df_summary$min_mm, 2)
   df_summary$max_mm <- round(df_summary$max_mm, 2)
   
   df_summary$B0 <- round(df_summary$B0, 3)
   df_summary$B0_lci <- round(df_summary$B0_lci, 3)
   df_summary$B0_uci <- round(df_summary$B0_uci, 3)
   
   df_summary$B1 <- round(df_summary$B1, 3)
   df_summary$B1_lci <- round(df_summary$B1_lci, 3)
   df_summary$B1_uci <- round(df_summary$B1_uci, 3)
   
   df_summary$SF <- round(df_summary$SF, 3)
   df_summary$SF_lci <- round(df_summary$SF_lci, 3)
   df_summary$SF_uci <- round(df_summary$SF_uci, 3)
   
   #Put CI in brackets behind each coefficient
   df_summary$B0 <- paste0(df_summary$B0, " [",   df_summary$B0_lci, ", ",   df_summary$B0_uci, "]")
   df_summary$B1 <- paste0(df_summary$B1, " [",   df_summary$B1_lci, ", ",   df_summary$B1_uci, "]")
   df_summary$SF <- paste0(df_summary$SF, " [",   df_summary$SF_lci, ", ",   df_summary$SF_uci, "]")
   
   #Remove unnecessary columns
   df_summary <- df_summary[,c("Order", "Taxon", "Stage", "N", "min_mm", "max_mm", "B0", "B1", "SF", "Weight_level", "Model_mm_mg", "Location")]
   
   #Remove rows without a length-biomass regression
   df_summary <- df_summary[!is.na(df_summary$Model_mm_mg),]
   
   #Combine Order and taxon in a single taxon column
   df_summary$taxon <- paste0(substr(df_summary$Order,1,4), ". ", df_summary$Taxon)
   
   #Change order of columns and change column names
   df_summary <- df_summary[,c("taxon", "N", "min_mm", "max_mm", "B0", "B1", "SF", "Weight_level", "Model_mm_mg", "Location")]
   colnames(df_summary) <- c("taxon", "n", "Min (mm)", "Max (mm)", "B0", "B1", "SF", "Level", "Model", "Location")
   
   #Sort by taxon
   df_summary <- df_summary[order(df_summary$taxon),]
   
   #Save dataframe
   write.csv(df_summary, file="Versluijs2022_Output_1-LengthBiomassRegressions_TableS1.csv", row.names=F)

 #(2): Plot the best model for all taxa for which data is only available for a single site
   
  #Note: commented below are the plots for taxa for which sample size < 8, or for which data is available for both sites (and they are plotted together further below)
   
  #Knipovich
    #p1 <- list_taxa[[1]][[7]]
    p2 <- list_taxa[[2]][[7]]
    p3 <- list_taxa[[3]][[7]]
    #p4 <- list_taxa[[4]][[7]]
    p5 <- list_taxa[[5]][[7]]
    p6 <- list_taxa[[6]][[7]]
    #p7 <- list_taxa[[7]][[7]]
    #p8 <- list_taxa[[8]][[7]]
    #p9 <- list_taxa[[9]][[7]]
    #p10 <- list_taxa[[10]][[7]]
    ##p11 <- list_taxa[[11]][[7]]
    #p12 <- list_taxa[[12]][[7]]
    p13 <- list_taxa[[13]][[7]]
    p14 <- list_taxa[[14]][[7]]
    #p15 <- list_taxa[[15]][[7]]
    ##p16 <- list_taxa[[16]][[7]] 
    p17 <- list_taxa[[17]][[7]]
   
  #Zackenberg:
    p18 <- list_taxa[[18]][[7]]
    p19 <- list_taxa[[19]][[7]]
    p20 <- list_taxa[[20]][[7]]
    #p21 <- list_taxa[[21]][[7]]
    p22 <- list_taxa[[22]][[7]]
    p23 <- list_taxa[[23]][[7]]
    #p24 <- list_taxa[[24]][[7]]
    p25 <- list_taxa[[25]][[7]]
    p26 <- list_taxa[[26]][[7]]
    p27 <- list_taxa[[27]][[7]]
    #p28 <- list_taxa[[28]][[7]]
    p29 <- list_taxa[[29]][[7]]
    #p30 <- list_taxa[[30]][[7]]
    #p31 <- list_taxa[[31]][[7]]
    #p32 <- list_taxa[[32]][[7]]
    p33 <- list_taxa[[33]][[7]]
    p34 <- list_taxa[[34]][[7]]
    p35 <- list_taxa[[35]][[7]]
    p36 <- list_taxa[[36]][[7]]
    p37 <- list_taxa[[37]][[7]]
    ##p38 <- list_taxa[[38]][[7]]
    ##p39 <- list_taxa[[39]][[7]]
    ##p40 <- list_taxa[[40]][[7]]
    #p41 <- list_taxa[[41]][[7]]
    ##p42 <- list_taxa[[42]][[7]]
    ##p43 <- list_taxa[[43]][[7]]
    p44 <- list_taxa[[44]][[7]]
    ##p45 <- list_taxa[[45]][[7]]
    ##p46 <- list_taxa[[46]][[7]]
   
   #Plot graphs  
    p_TAY <- grid.arrange(p2, p3, p5, p6, p13, p14, p17, ncol=3, nrow=3)   
    ggsave(p_TAY, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS2.png", width=18, height=14)  
    ggsave(p_TAY, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS2.pdf", width=18, height=14)  
    
    p_ZAC1 <- grid.arrange(p18, p19, p20, p22, p23, p25, p26, p27, p29, ncol=3, nrow=3)   
    ggsave(p_ZAC1, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS1a.png", width=18, height=14)  
    ggsave(p_ZAC1, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS1a.pdf", width=18, height=14)  
    
    p_ZAC2 <- grid.arrange(p33, p34, p36, p37, p44, ncol=3, nrow=3)   
    ggsave(p_ZAC2, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS1b.png", width=18, height=14)  
    ggsave(p_ZAC2, file="Versluijs2022_Output_1-LengthBiomassRegressions_FigS1b.pdf", width=18, height=14)  
   
 #(3): Plot data for Knipovich and Zackenberg in the same plot
    
  #Select for which taxa (with n>=20) there is data available for both ZAC and TAY  
   tmp <- df_full %>% group_by(Area, order, taxon, weight_level) %>% dplyr::summarize(N=n()) 
   tmp <- tmp[tmp$N >= 20,]
   tmp$name <- paste0(substr(tmp$order, 1, 3), "_" , substr(tmp$taxon, 1, 3), "_" , substr(tmp$weight_level, 1, 3))
   index_TAYZAC <- names(which(table(tmp$name)>1))  ;  rm(tmp)
   
   #Create an empty list for storing the plots:
   l_plots_combined <- rep(list(), length(index_TAYZAC))

   #Loop through all taxa for which data on both TAY and ZAC is available with the same weight_level
    for(i in index_TAYZAC){

     #Save plot in list
     l_plots_combined[[which(i == index_TAYZAC)]] <- local({
     
       #Store variables in local environment
        list_taxa=list_taxa
        i=i
       
       #Calculate two indicator variables 'j' and 'k' that indicate which entries in list_taxa correspond to the current taxa 'i' (i.e. one for ZAC and one for KNP)
        j = which(lapply(list_taxa, function(l) substr(l[[1]], 5, 15)) == i)[1]
        k = which(lapply(list_taxa, function(l) substr(l[[1]], 5, 15)) == i)[2]
        
       #Select required data for index j and k  
        df_taxon_j <- list_taxa[[j]][[2]]
        mod_predict_j <- list_taxa[[j]][[13]]
        boot_predict_CI_j <- list_taxa[[j]][[14]]
        boot_predict_j <- list_taxa[[j]][[15]]
        
        df_taxon_k <- list_taxa[[k]][[2]]
        mod_predict_k <- list_taxa[[k]][[13]]
        boot_predict_CI_k <- list_taxa[[k]][[14]]
        boot_predict_k <- list_taxa[[k]][[15]]

       #Create title
        title <- paste0(df_taxon_j$order[1], " ", df_taxon_j$taxon[1])  
        
        print(paste0("j=", df_taxon_j$Area[1]))
        print(paste0("k=", df_taxon_k$Area[1]))
     
       #Create plot

        #If bootstrapping data is available for both dataset j and k:
        if((any(!is.na(boot_predict_CI_j)) & any(!is.na(boot_predict_CI_k)))){

          p_taxon <- ggplot()+
            geom_line(data=boot_predict_j, aes(x=x, y=y, group=sim), col="blue", lty=1, lwd=0.75, alpha=0.01)+
            #geom_ribbon(aes(x=mod_predict_j$x, ymin=boot_predict_CI_j[,1], ymax=boot_predict_CI_j[,2]), colour=NA, alpha=0.75, fill="grey75")+
            geom_point(data=df_taxon_j, aes(x=length_mm, y=weight_mg), col="blue", alpha=0.25, pch=1, size=2, stroke=1.25)+
            geom_line(data=mod_predict_j, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
            geom_line(aes(x=mod_predict_j$x, y=boot_predict_CI_j[,1]), col="black", lty=3, lwd=1, alpha=1)+
            geom_line(aes(x=mod_predict_j$x, y=boot_predict_CI_j[,2]), col="black", lty=3, lwd=1, alpha=1)+
            geom_line(data=boot_predict_k, aes(x=x, y=y, group=sim), col="red", lty=1, lwd=0.75, alpha=0.01)+
            #geom_ribbon(aes(x=mod_predict_k$x, ymin=boot_predict_CI_k[,1], ymax=boot_predict_CI_k[,2]), colour=NA, alpha=0.75, fill="grey75")+
            geom_point(data=df_taxon_k, aes(x=length_mm, y=weight_mg), col="red", alpha=0.25, pch=2, size=2, stroke=1.25)+
            geom_line(data=mod_predict_k, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
            geom_line(aes(x=mod_predict_k$x, y=boot_predict_CI_k[,1]), col="black", lty=3, lwd=1, alpha=1)+
            geom_line(aes(x=mod_predict_k$x, y=boot_predict_CI_k[,2]), col="black", lty=3, lwd=1, alpha=1)+
            scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
            scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
            xlab("Bodylength (mm)")+
            ylab("Bodymass (mg)")+
            ggtitle(title)+
            theme_tom()

         }

        #If bootstrapping data is not available for either dataset j or k:
        if((!any(!is.na(boot_predict_CI_j)) | !any(!is.na(boot_predict_CI_k)))){
          
           p_taxon <- ggplot()+
            geom_line(data=mod_predict_j, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
            geom_point(data=df_taxon_j, aes(x=length_mm, y=weight_mg), col="blue", alpha=0.25, pch=1, size=2, stroke=1.25)+
            geom_line(data=mod_predict_k, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
            geom_point(data=df_taxon_k, aes(x=length_mm, y=weight_mg), col="red", alpha=0.25, pch=2, size=2, stroke=1.25)+
            scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
            scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
            xlab("Bodylength (mm)")+
            ylab("Bodymass (mg)")+
            ggtitle(title)+
            theme_tom()
          }
        
       #Output plot to global environment
        print(p_taxon)
     
     })
     
     }
    
   #The indicator 'index_TAYZAC' does not include Sciaridae because data for ZAC is based on averages and for TAY it is based on individual
   #weight measurements. We decided to manually add this plot and clearly state this detail in the figure caption.
    l_plots_combined[[8]] <- local({
       
       #Store variables in local environment
       list_taxa=list_taxa
       i="Dip_Sci"
       
       #Calculate index j and k for selecting the appropriate data from list_taxa
       j = which(lapply(list_taxa, function(l) substr(l[[1]], 5, 11)) == i)[1]
       k = which(lapply(list_taxa, function(l) substr(l[[1]], 5, 11)) == i)[2]
       
       #Select required data for index j and k  
       df_taxon_j <- list_taxa[[j]][[2]]
       mod_predict_j <- list_taxa[[j]][[13]]
       boot_predict_CI_j <- list_taxa[[j]][[14]]
       boot_predict_j <- list_taxa[[j]][[15]]
       
       df_taxon_k <- list_taxa[[k]][[2]]
       mod_predict_k <- list_taxa[[k]][[13]]
       boot_predict_CI_k <- list_taxa[[k]][[14]]
       boot_predict_k <- list_taxa[[k]][[15]]
       
       #Create title
       title <- paste0(df_taxon_j$order[1], " ", df_taxon_j$taxon[1])  
       
       print(paste0("j=", df_taxon_j$Area[1]))
       print(paste0("k=", df_taxon_k$Area[1]))
       
       #Create plot
       
       #If bootstrapping data is available for both dataset j and k:
       if((any(!is.na(boot_predict_CI_j)) & any(!is.na(boot_predict_CI_k)))){
         
         p_taxon <- ggplot()+
           geom_line(data=boot_predict_j, aes(x=x, y=y, group=sim), col="blue", lty=1, lwd=0.75, alpha=0.01)+
           #geom_ribbon(aes(x=mod_predict_j$x, ymin=boot_predict_CI_j[,1], ymax=boot_predict_CI_j[,2]), colour=NA, alpha=0.75, fill="grey75")+
           geom_point(data=df_taxon_j, aes(x=length_mm, y=weight_mg), col="blue", alpha=0.25, pch=1, size=2, stroke=1.25)+
           geom_line(data=mod_predict_j, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
           geom_line(aes(x=mod_predict_j$x, y=boot_predict_CI_j[,1]), col="black", lty=3, lwd=1, alpha=1)+
           geom_line(aes(x=mod_predict_j$x, y=boot_predict_CI_j[,2]), col="black", lty=3, lwd=1, alpha=1)+
           geom_line(data=boot_predict_k, aes(x=x, y=y, group=sim), col="red", lty=1, lwd=0.75, alpha=0.01)+
           #geom_ribbon(aes(x=mod_predict_k$x, ymin=boot_predict_CI_k[,1], ymax=boot_predict_CI_k[,2]), colour=NA, alpha=0.75, fill="grey75")+
           geom_point(data=df_taxon_k, aes(x=length_mm, y=weight_mg), col="red", alpha=0.25, pch=2, size=2, stroke=1.25)+
           geom_line(data=mod_predict_k, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
           geom_line(aes(x=mod_predict_k$x, y=boot_predict_CI_k[,1]), col="black", lty=3, lwd=1, alpha=1)+
           geom_line(aes(x=mod_predict_k$x, y=boot_predict_CI_k[,2]), col="black", lty=3, lwd=1, alpha=1)+
           scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
           scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
           xlab("Bodylength (mm)")+
           ylab("Bodymass (mg)")+
           ggtitle(title)+
           theme_tom()
         
       }
       
       #If bootstrapping data is not available for either dataset j or k:
       if((!any(!is.na(boot_predict_CI_j)) | !any(!is.na(boot_predict_CI_k)))){
         
         p_taxon <- ggplot()+
           geom_line(data=mod_predict_j, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
           geom_point(data=df_taxon_j, aes(x=length_mm, y=weight_mg), col="blue", alpha=0.25, pch=1, size=2, stroke=1.25)+
           geom_line(data=mod_predict_k, aes(x, y), col="black", lty=2, lwd=1, alpha=1)+
           geom_point(data=df_taxon_k, aes(x=length_mm, y=weight_mg), col="red", alpha=0.25, pch=2, size=2, stroke=1.25)+
           scale_x_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
           scale_y_continuous(trans = scales::log_trans(), breaks = scales::trans_breaks('log', n=4, function(x) round(exp(1)^x, 2)))+  #Rounded!
           xlab("Bodylength (mm)")+
           ylab("Bodymass (mg)")+
           ggtitle(title)+
           theme_tom()
       }
       
       #Output plot to global environment
       print(p_taxon)
       
     })
   
   #Store plots
   pc1 <- l_plots_combined[[1]]
   pc2 <- l_plots_combined[[2]]
   pc3 <- l_plots_combined[[3]]
   pc4 <- l_plots_combined[[4]]
   pc5 <- l_plots_combined[[5]]
   pc6 <- l_plots_combined[[6]]
   pc7 <- l_plots_combined[[7]]
   pc8 <- l_plots_combined[[8]]
   
   #Save plots
   p_combined <- grid.arrange(pc1, pc2, pc3, pc4, pc5, pc6, pc8, pc7, ncol=3, nrow=3)   
   ggsave(p_combined, file="Versluijs2022_Output_1-LengthBiomassRegressions_Fig1.png", width=18, height=14)  
   ggsave(p_combined, file="Versluijs2022_Output_1-LengthBiomassRegressions_Fig1.pdf", width=18, height=14)  
   
###################################################################################################################################
###################################################################################################################################
###################################################################################################################################
   